"""Lazy ZIP over HTTP"""

from __future__ import annotations

__all__ = ["HTTPRangeRequestUnsupported", "dist_from_wheel_url"]

from bisect import bisect_left, bisect_right
from collections.abc import Generator
from contextlib import contextmanager
from tempfile import NamedTemporaryFile
from typing import Any
from zipfile import BadZipFile, ZipFile

from pip._vendor.packaging.utils import NormalizedName
from pip._vendor.requests.models import CONTENT_CHUNK_SIZE, Response

from pip._internal.metadata import BaseDistribution, MemoryWheel, get_wheel_distribution
from pip._internal.network.session import PipSession
from pip._internal.network.utils import HEADERS, raise_for_status, response_chunks


class HTTPRangeRequestUnsupported(Exception):
    pass


def dist_from_wheel_url(
    name: NormalizedName, url: str, session: PipSession
) -> BaseDistribution:
    """Return a distribution object from the given wheel URL.

    This uses HTTP range requests to only fetch the portion of the wheel
    containing metadata, just enough for the object to be constructed.
    If such requests are not supported, HTTPRangeRequestUnsupported
    is raised.
    """
    with LazyZipOverHTTP(url, session) as zf:
        # For read-only ZIP files, ZipFile only needs methods read,
        # seek, seekable and tell, not the whole IO protocol.
        wheel = MemoryWheel(zf.name, zf)  # type: ignore
        # After context manager exit, wheel.name
        # is an invalid file by intention.
        return get_wheel_distribution(wheel, name)


class LazyZipOverHTTP:
    """File-like object mapped to a ZIP file over HTTP.

    This uses HTTP range requests to lazily fetch the file's content,
    which is supposed to be fed to ZipFile.  If such requests are not
    supported by the server, raise HTTPRangeRequestUnsupported
    during initialization.
    """

    def __init__(
        self, url: str, session: PipSession, chunk_size: int = CONTENT_CHUNK_SIZE
    ) -> None:
        head = session.head(url, headers=HEADERS)
        raise_for_status(head)
        assert head.status_code == 200
        self._session, self._url, self._chunk_size = session, url, chunk_size
        self._length = int(head.headers["Content-Length"])
        self._file = NamedTemporaryFile()
        self.truncate(self._length)
        self._left: list[int] = []
        self._right: list[int] = []
        if "bytes" not in head.headers.get("Accept-Ranges", "none"):
            raise HTTPRangeRequestUnsupported("range request is not supported")
        self._check_zip()

    @property
    def mode(self) -> str:
        """Opening mode, which is always rb."""
        return "rb"

    @property
    def name(self) -> str:
        """Path to the underlying file."""
        return self._file.name

    def seekable(self) -> bool:
        """Return whether random access is supported, which is True."""
        return True

    def close(self) -> None:
        """Close the file."""
        self._file.close()

    @property
    def closed(self) -> bool:
        """Whether the file is closed."""
        return self._file.closed

    def read(self, size: int = -1) -> bytes:
        """Read up to size bytes from the object and return them.

        As a convenience, if size is unspecified or -1,
        all bytes until EOF are returned.  Fewer than
        size bytes may be returned if EOF is reached.
        """
        download_size = max(size, self._chunk_size)
        start, length = self.tell(), self._length
        stop = length if size < 0 else min(start + download_size, length)
        start = max(0, stop - download_size)
        self._download(start, stop - 1)
        return self._file.read(size)

    def readable(self) -> bool:
        """Return whether the file is readable, which is True."""
        return True

    def seek(self, offset: int, whence: int = 0) -> int:
        """Change stream position and return the new absolute position.

        Seek to offset relative position indicated by whence:
        * 0: Start of stream (the default).  pos should be >= 0;
        * 1: Current position - pos may be negative;
        * 2: End of stream - pos usually negative.
        """
        return self._file.seek(offset, whence)

    def tell(self) -> int:
        """Return the current position."""
        return self._file.tell()

    def truncate(self, size: int | None = None) -> int:
        """Resize the stream to the given size in bytes.

        If size is unspecified resize to the current position.
        The current stream position isn't changed.

        Return the new file size.
        """
        return self._file.truncate(size)

    def writable(self) -> bool:
        """Return False."""
        return False

    def __enter__(self) -> LazyZipOverHTTP:
        self._file.__enter__()
        return self

    def __exit__(self, *exc: Any) -> None:
        self._file.__exit__(*exc)

    @contextmanager
    def _stay(self) -> Generator[None, None, None]:
        """Return a context manager keeping the position.

        At the end of the block, seek back to original position.
        """
        pos = self.tell()
        try:
            yield
        finally:
            self.seek(pos)

    def _check_zip(self) -> None:
        """Check and download until the file is a valid ZIP."""
        end = self._length - 1
        for start in reversed(range(0, end, self._chunk_size)):
            self._download(start, end)
            with self._stay():
                try:
                    # For read-only ZIP files, ZipFile only needs
                    # methods read, seek, seekable and tell.
                    ZipFile(self)
                except BadZipFile:
                    pass
                else:
                    break

    def _stream_response(
        self, start: int, end: int, base_headers: dict[str, str] = HEADERS
    ) -> Response:
        """Return HTTP response to a range request from start to end."""
        headers = base_headers.copy()
        headers["Range"] = f"bytes={start}-{end}"
        # TODO: Get range requests to be correctly cached
        headers["Cache-Control"] = "no-cache"
        return self._session.get(self._url, headers=headers, stream=True)

    def _merge(
        self, start: int, end: int, left: int, right: int
    ) -> Generator[tuple[int, int], None, None]:
        """Return a generator of intervals to be fetched.

        Args:
            start (int): Start of needed interval
            end (int): End of needed interval
            left (int): Index of first overlapping downloaded data
            right (int): Index after last overlapping downloaded data
        """
        lslice, rslice = self._left[left:right], self._right[left:right]
        i = start = min([start] + lslice[:1])
        end = max([end] + rslice[-1:])
        for j, k in zip(lslice, rslice):
            if j > i:
                yield i, j - 1
            i = k + 1
        if i <= end:
            yield i, end
        self._left[left:right], self._right[left:right] = [start], [end]

    def _download(self, start: int, end: int) -> None:
        """Download bytes from start to end inclusively."""
        with self._stay():
            left = bisect_left(self._right, start)
            right = bisect_right(self._left, end)
            for start, end in self._merge(start, end, left, right):
                response = self._stream_response(start, end)
                response.raise_for_status()
                self.seek(start)
                for chunk in response_chunks(response, self._chunk_size):
                    self._file.write(chunk)



# system configuration generated and used by the sysconfig module
build_time_vars = {
    "ABIFLAGS": "",
    "ABI_THREAD": "",
    "AC_APPLE_UNIVERSAL_BUILD": 0,
    "AIX_BUILDDATE": 0,
    "AIX_GENUINE_CPLUSPLUS": 0,
    "ALIGNOF_LONG": 8,
    "ALIGNOF_MAX_ALIGN_T": 16,
    "ALIGNOF_SIZE_T": 8,
    "ALT_SOABI": 0,
    "ANDROID_API_LEVEL": 0,
    "APP_STORE_COMPLIANCE_PATCH": "",
    "AR": "/tools/llvm/bin/llvm-ar",
    "ARFLAGS": "rcs",
    "BASECFLAGS": "-fno-strict-overflow -Wsign-compare -Wunreachable-code",
    "BASECPPFLAGS": "",
    "BASEMODLIBS": "",
    "BINDIR": "/install/bin",
    "BINLIBDEST": "/install/lib/python3.13",
    "BLDLIBRARY": "-L. -lpython3.13",
    "BLDSHARED": "clang -pthread -shared -Wl,-z,noexecstack  -Wl,--exclude-libs,ALL ",
    "BOOTSTRAP_HEADERS": "\\",
    "BUILDEXE": "",
    "BUILDPYTHON": "python",
    "BUILD_GNU_TYPE": "x86_64-unknown-linux-gnu",
    "BUILD_SCRIPTS_DIR": "build/scripts-3.13",
    "BYTESTR_DEPS": "\\",
    "CC": "clang -pthread",
    "CCSHARED": "-fPIC",
    "CFLAGS": "-fno-strict-overflow -Wsign-compare -Wunreachable-code -DNDEBUG -g -O3 -Wall -O3  -fno-omit-frame-pointer -mno-omit-leaf-frame-pointer  -fPIC  ",
    "CFLAGSFORSHARED": "-fPIC",
    "CFLAGS_ALIASING": "-fno-strict-aliasing",
    "CODECS_COMMON_HEADERS": "./Modules/cjkcodecs/multibytecodec.h ./Modules/cjkcodecs/cjkcodecs.h",
    "COMPILEALL_OPTS": "-j0",
    "CONFIGFILES": "configure configure.ac acconfig.h pyconfig.h.in Makefile.pre.in",
    "CONFIGURE_CFLAGS": "-O3  -fno-omit-frame-pointer -mno-omit-leaf-frame-pointer  -fPIC  ",
    "CONFIGURE_CFLAGS_NODIST": "-D_Py_TIER2=3 -D_Py_JIT -flto=thin -g -fno-pie -std=c11 -Wextra -Wno-unused-parameter -Wno-missing-field-initializers -Wstrict-prototypes -Werror=implicit-function-declaration -fvisibility=hidden",
    "CONFIGURE_CPPFLAGS": "-O3  -fno-omit-frame-pointer -mno-omit-leaf-frame-pointer  -fPIC  ",
    "CONFIGURE_LDFLAGS": "-Wl,-z,noexecstack  -Wl,--exclude-libs,ALL ",
    "CONFIGURE_LDFLAGS_NODIST": "-flto=thin -g -Wl,--emit-relocs",
    "CONFIGURE_LDFLAGS_NOLTO": "-flto=thin",
    "CONFIG_ARGS": "'--build=x86_64-unknown-linux-gnu' '--host=x86_64-unknown-linux-gnu' '--prefix=/install' '--with-openssl=/tools/deps' '--with-system-expat' '--with-system-libmpdec' '--without-ensurepip' '--with-readline=editline' '--enable-static-libpython-for-interpreter' '--enable-shared' '--with-mimalloc' '--enable-optimizations' '--enable-bolt' '--enable-experimental-jit=yes-off' '--with-lto' '--with-build-python=/tools/host/bin/python3.13' '--with-dbmliborder=bdb' 'build_alias=x86_64-unknown-linux-gnu' 'host_alias=x86_64-unknown-linux-gnu' 'PKG_CONFIG=pkg-config --static' 'PKG_CONFIG_PATH=/tools/deps/share/pkgconfig:/tools/deps/lib/pkgconfig' 'CC=clang' 'CFLAGS=-O3  -fno-omit-frame-pointer -mno-omit-leaf-frame-pointer  -fPIC  ' 'LDFLAGS=-Wl,-z,noexecstack  -Wl,--exclude-libs,ALL ' 'CPPFLAGS=-O3  -fno-omit-frame-pointer -mno-omit-leaf-frame-pointer  -fPIC  ' 'PROFILE_TASK=-m test --pgo -j 4 --ignore test.test_bytes.BytesTest.test_from_format'",
    "CONFINCLUDEDIR": "/install/include",
    "CONFINCLUDEPY": "/install/include/python3.13",
    "COREPYTHONPATH": "",
    "COVERAGE_INFO": "/build/Python-3.13.14/coverage.info",
    "COVERAGE_LCOV_OPTIONS": "--rc lcov_branch_coverage=1",
    "COVERAGE_REPORT": "/build/Python-3.13.14/lcov-report",
    "COVERAGE_REPORT_OPTIONS": "--rc lcov_branch_coverage=1 --branch-coverage --title \"CPython 3.13 LCOV report [commit $(shell )]\"",
    "CPPFLAGS": "-I. -I./Include -O3  -fno-omit-frame-pointer -mno-omit-leaf-frame-pointer  -fPIC  ",
    "CXX": "clang++ -pthread",
    "DESTDIRS": "/install /install/lib /install/lib/python3.13 /install/lib/python3.13/lib-dynload",
    "DESTLIB": "/install/lib/python3.13",
    "DESTPATH": "",
    "DESTSHARED": "/install/lib/python3.13/lib-dynload",
    "DFLAGS": "",
    "DIRMODE": 755,
    "DIST": "README.rst ChangeLog configure configure.ac acconfig.h pyconfig.h.in Makefile.pre.in Include Lib Misc Ext-dummy",
    "DISTDIRS": "Include Lib Misc Ext-dummy",
    "DISTFILES": "README.rst ChangeLog configure configure.ac acconfig.h pyconfig.h.in Makefile.pre.in",
    "DLINCLDIR": ".",
    "DLLLIBRARY": "",
    "DOUBLE_IS_ARM_MIXED_ENDIAN_IEEE754": 0,
    "DOUBLE_IS_BIG_ENDIAN_IEEE754": 0,
    "DOUBLE_IS_LITTLE_ENDIAN_IEEE754": 1,
    "DSYMUTIL": "",
    "DSYMUTIL_PATH": "",
    "DTRACE": "",
    "DTRACE_DEPS": "\\",
    "DTRACE_HEADERS": "",
    "DTRACE_OBJS": "",
    "DYNLOADFILE": "dynload_shlib.o",
    "ENABLE_IPV6": 1,
    "ENSUREPIP": "no",
    "EXE": "",
    "EXEMODE": 755,
    "EXENAME": "/install/bin/python3.13",
    "EXPORTSFROM": "",
    "EXPORTSYMS": "",
    "EXTRATESTOPTS": "",
    "EXTRA_CFLAGS": "",
    "EXT_SUFFIX": ".cpython-313-x86_64-linux-gnu.so",
    "FILEMODE": 644,
    "FREEZE_MODULE": "/tools/host/bin/python3.13 ./Programs/_freeze_module.py",
    "FREEZE_MODULE_BOOTSTRAP": "/tools/host/bin/python3.13 ./Programs/_freeze_module.py",
    "FREEZE_MODULE_BOOTSTRAP_DEPS": "./Programs/_freeze_module.py",
    "FREEZE_MODULE_DEPS": "./Programs/_freeze_module.py",
    "FROZEN_FILES_IN": "\\",
    "FROZEN_FILES_OUT": "\\",
    "GETPGRP_HAVE_ARG": 0,
    "GITBRANCH": "",
    "GITTAG": "",
    "GITVERSION": "",
    "GNULD": "yes",
    "HAVE_ACCEPT": 1,
    "HAVE_ACCEPT4": 1,
    "HAVE_ACOSH": 1,
    "HAVE_ADDRINFO": 1,
    "HAVE_ALARM": 1,
    "HAVE_ALIGNED_REQUIRED": 0,
    "HAVE_ALLOCA_H": 1,
    "HAVE_ALTZONE": 0,
    "HAVE_ASINH": 1,
    "HAVE_ASM_TYPES_H": 1,
    "HAVE_ATANH": 1,
    "HAVE_BIND": 1,
    "HAVE_BIND_TEXTDOMAIN_CODESET": 1,
    "HAVE_BLUETOOTH_BLUETOOTH_H": 0,
    "HAVE_BLUETOOTH_H": 0,
    "HAVE_BROKEN_MBSTOWCS": 0,
    "HAVE_BROKEN_NICE": 0,
    "HAVE_BROKEN_PIPE_BUF": 0,
    "HAVE_BROKEN_POLL": 0,
    "HAVE_BROKEN_POSIX_SEMAPHORES": 0,
    "HAVE_BROKEN_PTHREAD_SIGMASK": 0,
    "HAVE_BROKEN_SEM_GETVALUE": 0,
    "HAVE_BROKEN_UNSETENV": 0,
    "HAVE_BUILTIN_ATOMIC": 1,
    "HAVE_BZLIB_H": 1,
    "HAVE_CHFLAGS": 0,
    "HAVE_CHMOD": 1,
    "HAVE_CHOWN": 1,
    "HAVE_CHROOT": 1,
    "HAVE_CLOCK": 1,
    "HAVE_CLOCK_GETRES": 1,
    "HAVE_CLOCK_GETTIME": 1,
    "HAVE_CLOCK_NANOSLEEP": 1,
    "HAVE_CLOCK_SETTIME": 1,
    "HAVE_CLOCK_T": 1,
    "HAVE_CLOSEFROM": 0,
    "HAVE_CLOSE_RANGE": 0,
    "HAVE_COMPUTED_GOTOS": 1,
    "HAVE_CONFSTR": 1,
    "HAVE_CONIO_H": 0,
    "HAVE_CONNECT": 1,
    "HAVE_COPY_FILE_RANGE": 0,
    "HAVE_CTERMID": 1,
    "HAVE_CTERMID_R": 0,
    "HAVE_CURSES_FILTER": 1,
    "HAVE_CURSES_H": 1,
    "HAVE_CURSES_HAS_KEY": 1,
    "HAVE_CURSES_IMMEDOK": 1,
    "HAVE_CURSES_IS_PAD": 1,
    "HAVE_CURSES_IS_TERM_RESIZED": 1,
    "HAVE_CURSES_RESIZETERM": 1,
    "HAVE_CURSES_RESIZE_TERM": 1,
    "HAVE_CURSES_SYNCOK": 1,
    "HAVE_CURSES_TYPEAHEAD": 1,
    "HAVE_CURSES_USE_ENV": 1,
    "HAVE_CURSES_WCHGAT": 1,
    "HAVE_DB_H": 1,
    "HAVE_DECL_RTLD_DEEPBIND": 1,
    "HAVE_DECL_RTLD_GLOBAL": 1,
    "HAVE_DECL_RTLD_LAZY": 1,
    "HAVE_DECL_RTLD_LOCAL": 1,
    "HAVE_DECL_RTLD_MEMBER": 0,
    "HAVE_DECL_RTLD_NODELETE": 1,
    "HAVE_DECL_RTLD_NOLOAD": 1,
    "HAVE_DECL_RTLD_NOW": 1,
    "HAVE_DECL_TZNAME": 0,
    "HAVE_DECL_UT_NAMESIZE": 1,
    "HAVE_DEVICE_MACROS": 1,
    "HAVE_DEV_PTC": 0,
    "HAVE_DEV_PTMX": 1,
    "HAVE_DIRECT_H": 0,
    "HAVE_DIRENT_D_TYPE": 1,
    "HAVE_DIRENT_H": 1,
    "HAVE_DIRFD": 1,
    "HAVE_DLADDR": 1,
    "HAVE_DLFCN_H": 1,
    "HAVE_DLOPEN": 1,
    "HAVE_DUP": 1,
    "HAVE_DUP2": 1,
    "HAVE_DUP3": 1,
    "HAVE_DYLD_SHARED_CACHE_CONTAINS_PATH": 0,
    "HAVE_DYNAMIC_LOADING": 1,
    "HAVE_EDITLINE_READLINE_H": 0,
    "HAVE_ENDIAN_H": 1,
    "HAVE_EPOLL": 1,
    "HAVE_EPOLL_CREATE1": 1,
    "HAVE_ERF": 1,
    "HAVE_ERFC": 1,
    "HAVE_ERRNO_H": 1,
    "HAVE_EVENTFD": 1,
    "HAVE_EXECV": 1,
    "HAVE_EXPLICIT_BZERO": 0,
    "HAVE_EXPLICIT_MEMSET": 0,
    "HAVE_EXPM1": 1,
    "HAVE_FACCESSAT": 1,
    "HAVE_FCHDIR": 1,
    "HAVE_FCHMOD": 1,
    "HAVE_FCHMODAT": 1,
    "HAVE_FCHOWN": 1,
    "HAVE_FCHOWNAT": 1,
    "HAVE_FCNTL_H": 1,
    "HAVE_FDATASYNC": 1,
    "HAVE_FDOPENDIR": 1,
    "HAVE_FDWALK": 0,
    "HAVE_FEXECVE": 1,
    "HAVE_FFI_CLOSURE_ALLOC": 1,
    "HAVE_FFI_PREP_CIF_VAR": 1,
    "HAVE_FFI_PREP_CLOSURE_LOC": 1,
    "HAVE_FLOCK": 1,
    "HAVE_FORK": 1,
    "HAVE_FORK1": 0,
    "HAVE_FORKPTY": 1,
    "HAVE_FPATHCONF": 1,
    "HAVE_FSEEK64": 0,
    "HAVE_FSEEKO": 1,
    "HAVE_FSTATAT": 1,
    "HAVE_FSTATVFS": 1,
    "HAVE_FSYNC": 1,
    "HAVE_FTELL64": 0,
    "HAVE_FTELLO": 1,
    "HAVE_FTIME": 1,
    "HAVE_FTRUNCATE": 1,
    "HAVE_FUTIMENS": 1,
    "HAVE_FUTIMES": 1,
    "HAVE_FUTIMESAT": 1,
    "HAVE_GAI_STRERROR": 1,
    "HAVE_GCC_ASM_FOR_MC68881": 0,
    "HAVE_GCC_ASM_FOR_X64": 1,
    "HAVE_GCC_ASM_FOR_X87": 1,
    "HAVE_GCC_UINT128_T": 1,
    "HAVE_GDBM_DASH_NDBM_H": 0,
    "HAVE_GDBM_H": 0,
    "HAVE_GDBM_NDBM_H": 0,
    "HAVE_GETADDRINFO": 1,
    "HAVE_GETC_UNLOCKED": 1,
    "HAVE_GETEGID": 1,
    "HAVE_GETENTROPY": 0,
    "HAVE_GETEUID": 1,
    "HAVE_GETGID": 1,
    "HAVE_GETGRENT": 1,
    "HAVE_GETGRGID": 1,
    "HAVE_GETGRGID_R": 1,
    "HAVE_GETGRNAM_R": 1,
    "HAVE_GETGROUPLIST": 1,
    "HAVE_GETGROUPS": 1,
    "HAVE_GETHOSTBYADDR": 1,
    "HAVE_GETHOSTBYNAME": 1,
    "HAVE_GETHOSTBYNAME_R": 1,
    "HAVE_GETHOSTBYNAME_R_3_ARG": 0,
    "HAVE_GETHOSTBYNAME_R_5_ARG": 0,
    "HAVE_GETHOSTBYNAME_R_6_ARG": 1,
    "HAVE_GETHOSTNAME": 1,
    "HAVE_GETITIMER": 1,
    "HAVE_GETLOADAVG": 1,
    "HAVE_GETLOGIN": 1,
    "HAVE_GETLOGIN_R": 1,
    "HAVE_GETNAMEINFO": 1,
    "HAVE_GETPAGESIZE": 1,
    "HAVE_GETPEERNAME": 1,
    "HAVE_GETPGID": 1,
    "HAVE_GETPGRP": 1,
    "HAVE_GETPID": 1,
    "HAVE_GETPPID": 1,
    "HAVE_GETPRIORITY": 1,
    "HAVE_GETPROTOBYNAME": 1,
    "HAVE_GETPWENT": 1,
    "HAVE_GETPWNAM_R": 1,
    "HAVE_GETPWUID": 1,
    "HAVE_GETPWUID_R": 1,
    "HAVE_GETRANDOM": 0,
    "HAVE_GETRANDOM_SYSCALL": 0,
    "HAVE_GETRESGID": 1,
    "HAVE_GETRESUID": 1,
    "HAVE_GETRUSAGE": 1,
    "HAVE_GETSERVBYNAME": 1,
    "HAVE_GETSERVBYPORT": 1,
    "HAVE_GETSID": 1,
    "HAVE_GETSOCKNAME": 1,
    "HAVE_GETSPENT": 1,
    "HAVE_GETSPNAM": 1,
    "HAVE_GETUID": 1,
    "HAVE_GETWD": 1,
    "HAVE_GLIBC_MEMMOVE_BUG": 0,
    "HAVE_GRANTPT": 1,
    "HAVE_GRP_H": 1,
    "HAVE_HSTRERROR": 1,
    "HAVE_HTOLE64": 1,
    "HAVE_IF_NAMEINDEX": 1,
    "HAVE_INET_ATON": 1,
    "HAVE_INET_NTOA": 1,
    "HAVE_INET_PTON": 1,
    "HAVE_INITGROUPS": 1,
    "HAVE_INTTYPES_H": 1,
    "HAVE_IO_H": 0,
    "HAVE_IPA_PURE_CONST_BUG": 0,
    "HAVE_KILL": 1,
    "HAVE_KILLPG": 1,
    "HAVE_KQUEUE": 0,
    "HAVE_LANGINFO_H": 1,
    "HAVE_LARGEFILE_SUPPORT": 0,
    "HAVE_LCHFLAGS": 0,
    "HAVE_LCHMOD": 0,
    "HAVE_LCHOWN": 1,
    "HAVE_LIBB2": 0,
    "HAVE_LIBDB": 1,
    "HAVE_LIBDL": 1,
    "HAVE_LIBDLD": 0,
    "HAVE_LIBIEEE": 0,
    "HAVE_LIBINTL_H": 1,
    "HAVE_LIBRESOLV": 0,
    "HAVE_LIBSENDFILE": 0,
    "HAVE_LIBSQLITE3": 1,
    "HAVE_LIBUTIL_H": 0,
    "HAVE_LINK": 1,
    "HAVE_LINKAT": 1,
    "HAVE_LINUX_AUXVEC_H": 1,
    "HAVE_LINUX_CAN_BCM_H": 1,
    "HAVE_LINUX_CAN_H": 1,
    "HAVE_LINUX_CAN_J1939_H": 0,
    "HAVE_LINUX_CAN_RAW_FD_FRAMES": 1,
    "HAVE_LINUX_CAN_RAW_H": 1,
    "HAVE_LINUX_CAN_RAW_JOIN_FILTERS": 0,
    "HAVE_LINUX_FS_H": 1,
    "HAVE_LINUX_LIMITS_H": 1,
    "HAVE_LINUX_MEMFD_H": 1,
    "HAVE_LINUX_NETLINK_H": 1,
    "HAVE_LINUX_QRTR_H": 0,
    "HAVE_LINUX_RANDOM_H": 1,
    "HAVE_LINUX_SOUNDCARD_H": 1,
    "HAVE_LINUX_TIPC_H": 1,
    "HAVE_LINUX_VM_SOCKETS_H": 1,
    "HAVE_LINUX_WAIT_H": 1,
    "HAVE_LISTEN": 1,
    "HAVE_LOCKF": 1,
    "HAVE_LOG1P": 1,
    "HAVE_LOG2": 1,
    "HAVE_LOGIN_TTY": 1,
    "HAVE_LONG_DOUBLE": 1,
    "HAVE_LSTAT": 1,
    "HAVE_LUTIMES": 1,
    "HAVE_LZMA_H": 0,
    "HAVE_MADVISE": 1,
    "HAVE_MAKEDEV": 1,
    "HAVE_MAXLOGNAME": 0,
    "HAVE_MBRTOWC": 1,
    "HAVE_MEMFD_CREATE": 0,
    "HAVE_MEMRCHR": 1,
    "HAVE_MINIX_CONFIG_H": 0,
    "HAVE_MKDIRAT": 1,
    "HAVE_MKFIFO": 1,
    "HAVE_MKFIFOAT": 1,
    "HAVE_MKNOD": 1,
    "HAVE_MKNODAT": 1,
    "HAVE_MKTIME": 1,
    "HAVE_MMAP": 1,
    "HAVE_MREMAP": 1,
    "HAVE_NANOSLEEP": 1,
    "HAVE_NCURSES": 0,
    "HAVE_NCURSESW": 0,
    "HAVE_NCURSESW_CURSES_H": 1,
    "HAVE_NCURSESW_NCURSES_H": 1,
    "HAVE_NCURSESW_PANEL_H": 1,
    "HAVE_NCURSES_CURSES_H": 0,
    "HAVE_NCURSES_H": 1,
    "HAVE_NCURSES_NCURSES_H": 0,
    "HAVE_NCURSES_PANEL_H": 0,
    "HAVE_NDBM_H": 0,
    "HAVE_NDIR_H": 0,
    "HAVE_NETCAN_CAN_H": 0,
    "HAVE_NETDB_H": 1,
    "HAVE_NETINET_IN_H": 1,
    "HAVE_NETLINK_NETLINK_H": 0,
    "HAVE_NETPACKET_PACKET_H": 1,
    "HAVE_NET_ETHERNET_H": 1,
    "HAVE_NET_IF_H": 1,
    "HAVE_NICE": 1,
    "HAVE_NON_UNICODE_WCHAR_T_REPRESENTATION": 0,
    "HAVE_OPENAT": 1,
    "HAVE_OPENDIR": 1,
    "HAVE_OPENPTY": 1,
    "HAVE_PANEL": 0,
    "HAVE_PANELW": 0,
    "HAVE_PANEL_H": 1,
    "HAVE_PATHCONF": 1,
    "HAVE_PAUSE": 1,
    "HAVE_PIPE": 1,
    "HAVE_PIPE2": 1,
    "HAVE_PLOCK": 0,
    "HAVE_POLL": 1,
    "HAVE_POLL_H": 1,
    "HAVE_POSIX_FADVISE": 1,
    "HAVE_POSIX_FALLOCATE": 1,
    "HAVE_POSIX_OPENPT": 1,
    "HAVE_POSIX_SPAWN": 1,
    "HAVE_POSIX_SPAWNP": 1,
    "HAVE_POSIX_SPAWN_FILE_ACTIONS_ADDCLOSEFROM_NP": 0,
    "HAVE_PREAD": 1,
    "HAVE_PREADV": 1,
    "HAVE_PREADV2": 0,
    "HAVE_PRLIMIT": 1,
    "HAVE_PROCESS_H": 0,
    "HAVE_PROCESS_VM_READV": 1,
    "HAVE_PROTOTYPES": 1,
    "HAVE_PTHREAD_CONDATTR_SETCLOCK": 1,
    "HAVE_PTHREAD_COND_TIMEDWAIT_RELATIVE_NP": 0,
    "HAVE_PTHREAD_DESTRUCTOR": 0,
    "HAVE_PTHREAD_GETCPUCLOCKID": 1,
    "HAVE_PTHREAD_H": 1,
    "HAVE_PTHREAD_INIT": 0,
    "HAVE_PTHREAD_KILL": 1,
    "HAVE_PTHREAD_SIGMASK": 1,
    "HAVE_PTHREAD_STUBS": 0,
    "HAVE_PTSNAME": 1,
    "HAVE_PTSNAME_R": 1,
    "HAVE_PTY_H": 1,
    "HAVE_PWRITE": 1,
    "HAVE_PWRITEV": 1,
    "HAVE_PWRITEV2": 0,
    "HAVE_READLINE_READLINE_H": 0,
    "HAVE_READLINK": 1,
    "HAVE_READLINKAT": 1,
    "HAVE_READV": 1,
    "HAVE_REALPATH": 1,
    "HAVE_RECVFROM": 1,
    "HAVE_RENAMEAT": 1,
    "HAVE_RL_APPEND_HISTORY": 1,
    "HAVE_RL_CATCH_SIGNAL": 1,
    "HAVE_RL_COMPDISP_FUNC_T": 1,
    "HAVE_RL_COMPLETION_APPEND_CHARACTER": 1,
    "HAVE_RL_COMPLETION_DISPLAY_MATCHES_HOOK": 1,
    "HAVE_RL_COMPLETION_MATCHES": 1,
    "HAVE_RL_COMPLETION_SUPPRESS_APPEND": 1,
    "HAVE_RL_PRE_INPUT_HOOK": 1,
    "HAVE_RL_RESIZE_TERMINAL": 1,
    "HAVE_RTPSPAWN": 0,
    "HAVE_SCHED_GET_PRIORITY_MAX": 1,
    "HAVE_SCHED_H": 1,
    "HAVE_SCHED_RR_GET_INTERVAL": 1,
    "HAVE_SCHED_SETAFFINITY": 1,
    "HAVE_SCHED_SETPARAM": 1,
    "HAVE_SCHED_SETSCHEDULER": 1,
    "HAVE_SEM_CLOCKWAIT": 0,
    "HAVE_SEM_GETVALUE": 1,
    "HAVE_SEM_OPEN": 1,
    "HAVE_SEM_TIMEDWAIT": 1,
    "HAVE_SEM_UNLINK": 1,
    "HAVE_SENDFILE": 1,
    "HAVE_SENDTO": 1,
    "HAVE_SETEGID": 1,
    "HAVE_SETEUID": 1,
    "HAVE_SETGID": 1,
    "HAVE_SETGROUPS": 1,
    "HAVE_SETHOSTNAME": 1,
    "HAVE_SETITIMER": 1,
    "HAVE_SETJMP_H": 1,
    "HAVE_SETLOCALE": 1,
    "HAVE_SETNS": 1,
    "HAVE_SETPGID": 1,
    "HAVE_SETPGRP": 1,
    "HAVE_SETPRIORITY": 1,
    "HAVE_SETREGID": 1,
    "HAVE_SETRESGID": 1,
    "HAVE_SETRESUID": 1,
    "HAVE_SETREUID": 1,
    "HAVE_SETSID": 1,
    "HAVE_SETSOCKOPT": 1,
    "HAVE_SETUID": 1,
    "HAVE_SETVBUF": 1,
    "HAVE_SHADOW_H": 1,
    "HAVE_SHM_OPEN": 1,
    "HAVE_SHM_UNLINK": 1,
    "HAVE_SHUTDOWN": 1,
    "HAVE_SIGACTION": 1,
    "HAVE_SIGALTSTACK": 1,
    "HAVE_SIGFILLSET": 1,
    "HAVE_SIGINFO_T_SI_BAND": 1,
    "HAVE_SIGINTERRUPT": 1,
    "HAVE_SIGNAL_H": 1,
    "HAVE_SIGPENDING": 1,
    "HAVE_SIGRELSE": 1,
    "HAVE_SIGTIMEDWAIT": 1,
    "HAVE_SIGWAIT": 1,
    "HAVE_SIGWAITINFO": 1,
    "HAVE_SNPRINTF": 1,
    "HAVE_SOCKADDR_ALG": 1,
    "HAVE_SOCKADDR_SA_LEN": 0,
    "HAVE_SOCKADDR_STORAGE": 1,
    "HAVE_SOCKET": 1,
    "HAVE_SOCKETPAIR": 1,
    "HAVE_SOCKLEN_T": 1,
    "HAVE_SPAWN_H": 1,
    "HAVE_SPLICE": 1,
    "HAVE_SSIZE_T": 1,
    "HAVE_STATVFS": 1,
    "HAVE_STAT_TV_NSEC": 1,
    "HAVE_STAT_TV_NSEC2": 0,
    "HAVE_STDINT_H": 1,
    "HAVE_STDIO_H": 1,
    "HAVE_STDLIB_H": 1,
    "HAVE_STD_ATOMIC": 1,
    "HAVE_STRFTIME": 1,
    "HAVE_STRINGS_H": 1,
    "HAVE_STRING_H": 1,
    "HAVE_STRLCPY": 0,
    "HAVE_STROPTS_H": 1,
    "HAVE_STRSIGNAL": 1,
    "HAVE_STRUCT_PASSWD_PW_GECOS": 1,
    "HAVE_STRUCT_PASSWD_PW_PASSWD": 1,
    "HAVE_STRUCT_STAT_ST_BIRTHTIME": 0,
    "HAVE_STRUCT_STAT_ST_BLKSIZE": 1,
    "HAVE_STRUCT_STAT_ST_BLOCKS": 1,
    "HAVE_STRUCT_STAT_ST_FLAGS": 0,
    "HAVE_STRUCT_STAT_ST_GEN": 0,
    "HAVE_STRUCT_STAT_ST_RDEV": 1,
    "HAVE_STRUCT_TM_TM_ZONE": 1,
    "HAVE_SYMLINK": 1,
    "HAVE_SYMLINKAT": 1,
    "HAVE_SYNC": 1,
    "HAVE_SYSCONF": 1,
    "HAVE_SYSEXITS_H": 1,
    "HAVE_SYSLOG_H": 1,
    "HAVE_SYSTEM": 1,
    "HAVE_SYS_AUDIOIO_H": 0,
    "HAVE_SYS_AUXV_H": 1,
    "HAVE_SYS_BSDTTY_H": 0,
    "HAVE_SYS_DEVPOLL_H": 0,
    "HAVE_SYS_DIR_H": 0,
    "HAVE_SYS_ENDIAN_H": 0,
    "HAVE_SYS_EPOLL_H": 1,
    "HAVE_SYS_EVENTFD_H": 1,
    "HAVE_SYS_EVENT_H": 0,
    "HAVE_SYS_FILE_H": 1,
    "HAVE_SYS_IOCTL_H": 1,
    "HAVE_SYS_KERN_CONTROL_H": 0,
    "HAVE_SYS_LOADAVG_H": 0,
    "HAVE_SYS_LOCK_H": 0,
    "HAVE_SYS_MEMFD_H": 0,
    "HAVE_SYS_MKDEV_H": 0,
    "HAVE_SYS_MMAN_H": 1,
    "HAVE_SYS_MODEM_H": 0,
    "HAVE_SYS_NDIR_H": 0,
    "HAVE_SYS_PARAM_H": 1,
    "HAVE_SYS_PIDFD_H": 0,
    "HAVE_SYS_POLL_H": 1,
    "HAVE_SYS_RANDOM_H": 0,
    "HAVE_SYS_RESOURCE_H": 1,
    "HAVE_SYS_SELECT_H": 1,
    "HAVE_SYS_SENDFILE_H": 1,
    "HAVE_SYS_SOCKET_H": 1,
    "HAVE_SYS_SOUNDCARD_H": 1,
    "HAVE_SYS_STATVFS_H": 1,
    "HAVE_SYS_STAT_H": 1,
    "HAVE_SYS_SYSCALL_H": 1,
    "HAVE_SYS_SYSMACROS_H": 1,
    "HAVE_SYS_SYS_DOMAIN_H": 0,
    "HAVE_SYS_TERMIO_H": 0,
    "HAVE_SYS_TIMERFD_H": 1,
    "HAVE_SYS_TIMES_H": 1,
    "HAVE_SYS_TIME_H": 1,
    "HAVE_SYS_TYPES_H": 1,
    "HAVE_SYS_UIO_H": 1,
    "HAVE_SYS_UN_H": 1,
    "HAVE_SYS_UTSNAME_H": 1,
    "HAVE_SYS_WAIT_H": 1,
    "HAVE_SYS_XATTR_H": 1,
    "HAVE_TCGETPGRP": 1,
    "HAVE_TCSETPGRP": 1,
    "HAVE_TEMPNAM": 1,
    "HAVE_TERMIOS_H": 1,
    "HAVE_TERM_H": 1,
    "HAVE_TIMEGM": 1,
    "HAVE_TIMERFD_CREATE": 1,
    "HAVE_TIMES": 1,
    "HAVE_TMPFILE": 1,
    "HAVE_TMPNAM": 1,
    "HAVE_TMPNAM_R": 1,
    "HAVE_TM_ZONE": 1,
    "HAVE_TRUNCATE": 1,
    "HAVE_TTYNAME_R": 1,
    "HAVE_TZNAME": 0,
    "HAVE_UMASK": 1,
    "HAVE_UNAME": 1,
    "HAVE_UNISTD_H": 1,
    "HAVE_UNLINKAT": 1,
    "HAVE_UNLOCKPT": 1,
    "HAVE_UNSHARE": 1,
    "HAVE_USABLE_WCHAR_T": 0,
    "HAVE_UTIL_H": 0,
    "HAVE_UTIMENSAT": 1,
    "HAVE_UTIMES": 1,
    "HAVE_UTIME_H": 1,
    "HAVE_UTMP_H": 1,
    "HAVE_UT_NAMESIZE": 1,
    "HAVE_UUID_CREATE": 0,
    "HAVE_UUID_ENC_BE": 0,
    "HAVE_UUID_GENERATE_TIME_SAFE": 1,
    "HAVE_UUID_GENERATE_TIME_SAFE_STABLE_MAC": 0,
    "HAVE_UUID_H": 0,
    "HAVE_UUID_UUID_H": 1,
    "HAVE_VFORK": 1,
    "HAVE_WAIT": 1,
    "HAVE_WAIT3": 1,
    "HAVE_WAIT4": 1,
    "HAVE_WAITID": 1,
    "HAVE_WAITPID": 1,
    "HAVE_WCHAR_H": 1,
    "HAVE_WCSCOLL": 1,
    "HAVE_WCSFTIME": 1,
    "HAVE_WCSXFRM": 1,
    "HAVE_WMEMCMP": 1,
    "HAVE_WORKING_TZSET": 1,
    "HAVE_WRITEV": 1,
    "HAVE_ZLIB_COPY": 1,
    "HAVE_ZLIB_H": 0,
    "HAVE__GETPTY": 0,
    "HAVE___UINT128_T": 1,
    "HOSTRUNNER": "",
    "HOST_GNU_TYPE": "x86_64-unknown-linux-gnu",
    "INCLDIRSTOMAKE": "/install/include /install/include /install/include/python3.13 /install/include/python3.13",
    "INCLUDEDIR": "/install/include",
    "INCLUDEPY": "/install/include/python3.13",
    "INSTALL": "/usr/bin/install -c",
    "INSTALL_DATA": "/usr/bin/install -c -m 644",
    "INSTALL_MIMALLOC": "yes",
    "INSTALL_PROGRAM": "/usr/bin/install -c",
    "INSTALL_SCRIPT": "/usr/bin/install -c",
    "INSTALL_SHARED": "/usr/bin/install -c -m 755",
    "INSTSONAME": "libpython3.13.so.1.0",
    "IO_H": "Modules/_io/_iomodule.h",
    "IO_OBJS": "\\",
    "IPHONEOS_DEPLOYMENT_TARGET": "",
    "JIT_DEPS": "\\",
    "LDCXXSHARED": "clang++ -pthread -shared -Wl,-z,noexecstack  -Wl,--exclude-libs,ALL ",
    "LDFLAGS": "-Wl,-z,noexecstack  -Wl,--exclude-libs,ALL ",
    "LDLIBRARY": "libpython3.13.so",
    "LDLIBRARYDIR": "",
    "LDSHARED": "clang -pthread -shared -Wl,-z,noexecstack  -Wl,--exclude-libs,ALL ",
    "LDVERSION": "3.13",
    "LIBC": "",
    "LIBDEST": "/install/lib/python3.13",
    "LIBDIR": "/install/lib",
    "LIBEXPAT_A": "Modules/expat/libexpat.a",
    "LIBEXPAT_CFLAGS": "-fno-strict-overflow -Wsign-compare -Wunreachable-code -DNDEBUG -g -O3 -Wall -O3  -fno-omit-frame-pointer -mno-omit-leaf-frame-pointer  -fPIC   -D_Py_TIER2=3 -D_Py_JIT -flto=thin -g -fno-pie -std=c11 -Wextra -Wno-unused-parameter -Wno-missing-field-initializers -Wstrict-prototypes -Werror=implicit-function-declaration -fvisibility=hidden -fprofile-instr-use=/build/Python-3.13.14/code.profclangd -I./Include/internal -I./Include/internal/mimalloc -I. -I./Include -O3  -fno-omit-frame-pointer -mno-omit-leaf-frame-pointer  -fPIC   -fPIC -fPIC",
    "LIBEXPAT_HEADERS": "\\",
    "LIBEXPAT_OBJS": "\\",
    "LIBHACL_CFLAGS": "-I./Modules/_hacl/include -D_BSD_SOURCE -D_DEFAULT_SOURCE -fno-strict-overflow -Wsign-compare -Wunreachable-code -DNDEBUG -g -O3 -Wall -O3  -fno-omit-frame-pointer -mno-omit-leaf-frame-pointer  -fPIC   -D_Py_TIER2=3 -D_Py_JIT -flto=thin -g -fno-pie -std=c11 -Wextra -Wno-unused-parameter -Wno-missing-field-initializers -Wstrict-prototypes -Werror=implicit-function-declaration -fvisibility=hidden -fprofile-instr-use=/build/Python-3.13.14/code.profclangd -I./Include/internal -I./Include/internal/mimalloc -I. -I./Include -O3  -fno-omit-frame-pointer -mno-omit-leaf-frame-pointer  -fPIC   -fPIC -fPIC",
    "LIBHACL_HEADERS": "\\",
    "LIBHACL_SHA2_A": "Modules/_hacl/libHacl_Hash_SHA2.a",
    "LIBHACL_SHA2_HEADERS": "\\",
    "LIBHACL_SHA2_OBJS": "\\",
    "LIBM": "-lm",
    "LIBMPDEC_A": "Modules/_decimal/libmpdec/libmpdec.a",
    "LIBMPDEC_CFLAGS": "  -fno-strict-overflow -Wsign-compare -Wunreachable-code -DNDEBUG -g -O3 -Wall -O3  -fno-omit-frame-pointer -mno-omit-leaf-frame-pointer  -fPIC   -D_Py_TIER2=3 -D_Py_JIT -flto=thin -g -fno-pie -std=c11 -Wextra -Wno-unused-parameter -Wno-missing-field-initializers -Wstrict-prototypes -Werror=implicit-function-declaration -fvisibility=hidden -fprofile-instr-use=/build/Python-3.13.14/code.profclangd -I./Include/internal -I./Include/internal/mimalloc -I. -I./Include -O3  -fno-omit-frame-pointer -mno-omit-leaf-frame-pointer  -fPIC   -fPIC -fPIC",
    "LIBMPDEC_HEADERS": "\\",
    "LIBMPDEC_OBJS": "\\",
    "LIBOBJDIR": "Python/",
    "LIBOBJS": "",
    "LIBPC": "/install/lib/pkgconfig",
    "LIBPL": "/install/lib/python3.13/config-3.13-x86_64-linux-gnu",
    "LIBPYTHON": "",
    "LIBRARY": "libpython3.13.a",
    "LIBRARY_DEPS": "libpython3.13.a libpython3.13.so libpython3.so",
    "LIBRARY_OBJS": "\\",
    "LIBRARY_OBJS_OMIT_FROZEN": "\\",
    "LIBS": "-lpthread -ldl  -lutil",
    "LIBSUBDIRS": "asyncio \\",
    "LINKCC": "clang -pthread -fno-pie -no-pie",
    "LINKFORSHARED": "-Xlinker -export-dynamic",
    "LINK_PYTHON_DEPS": "libpython3.13.a libpython3.13.so libpython3.so",
    "LINK_PYTHON_OBJS": "\\",
    "LIPO_32BIT_FLAGS": "",
    "LIPO_INTEL64_FLAGS": "",
    "LLVM_MERGED_DATA_FILE": "$(shell pwd)/code.profclangd",
    "LLVM_PROFDATA": "/tools/llvm/bin/llvm-profdata",
    "LLVM_PROF_ERR": "no",
    "LLVM_PROF_FILE": "LLVM_PROFILE_FILE=\"$(shell pwd)/code-%128m.profclangr\"",
    "LLVM_PROF_MERGER": "/tools/llvm/bin/llvm-profdata merge -output=\"$(shell pwd)/code.profclangd\" \"$(shell pwd)\"/*.profclangr",
    "LN": "ln",
    "LOCALMODLIBS": "-lbz2          -lffi -ldl  -lncursesw  -lpanelw -lncursesw   -lmpdec  -lexpat  -lcrypto -l:libatomic.a        -llzma       -lrt         -lsqlite3  -lssl -lcrypto -l:libatomic.a        -luuid      -lm    -lm   -lexpat  -ledit -lncursesw        -lz",
    "MACHDEP": "linux",
    "MACHDEP_OBJS": "",
    "MACHDESTLIB": "/install/lib/python3.13",
    "MACOSX_DEPLOYMENT_TARGET": "",
    "MAJOR_IN_MKDEV": 0,
    "MAJOR_IN_SYSMACROS": 1,
    "MAKESETUP": "./Modules/makesetup",
    "MANDIR": "/install/share/man",
    "MIMALLOC_HEADERS": "\\",
    "MKDIR_P": "/bin/mkdir -p",
    "MODBUILT_NAMES": "_ctypes_test  _dbm  _testbuffer  _testcapi  _testexternalinspection  _testimportmultiple  _testlimitedcapi  _testmultiphase  _testsinglephase  _tkinter  _asyncio  _bisect  _blake2  _bz2  _codecs_cn  _codecs_hk  _codecs_iso2022  _codecs_jp  _codecs_kr  _codecs_tw  _contextvars  _csv  _ctypes  _curses  _curses_panel  _datetime  _decimal  _elementtree  _hashlib  _heapq  _interpchannels  _interpqueues  _interpreters  _json  _lsprof  _lzma  _md5  _multibytecodec  _multiprocessing  _opcode  _pickle  _posixshmem  _posixsubprocess  _queue  _random  _sha1  _sha2  _sha3  _socket  _sqlite3  _ssl  _statistics  _struct  _suggestions  _sysconfig  _testinternalcapi  _typing  _uuid  _xxtestfuzz  _zoneinfo  array  binascii  cmath  fcntl  grp  math  mmap  pyexpat  readline  resource  select  syslog  termios  unicodedata  xxsubtype  zlib  atexit  faulthandler  posix  _signal  _tracemalloc  _codecs  _collections  errno  _io  itertools  _sre  _thread  time  _weakref  _abc  _functools  _locale  _operator  _stat  _symtable  pwd",
    "MODDISABLED_NAMES": "_gdbm  _scproxy  xx  xxlimited  xxlimited_35",
    "MODLIBS": "-lbz2          -lffi -ldl  -lncursesw  -lpanelw -lncursesw   -lmpdec  -lexpat  -lcrypto -l:libatomic.a        -llzma       -lrt         -lsqlite3  -lssl -lcrypto -l:libatomic.a        -luuid      -lm    -lm   -lexpat  -ledit -lncursesw        -lz",
    "MODOBJS": "Modules/_asynciomodule.o  Modules/_bisectmodule.o  Modules/_blake2/blake2module.o Modules/_blake2/blake2b_impl.o Modules/_blake2/blake2s_impl.o  Modules/_bz2module.o  Modules/cjkcodecs/_codecs_cn.o  Modules/cjkcodecs/_codecs_hk.o  Modules/cjkcodecs/_codecs_iso2022.o  Modules/cjkcodecs/_codecs_jp.o  Modules/cjkcodecs/_codecs_kr.o  Modules/cjkcodecs/_codecs_tw.o  Modules/_contextvarsmodule.o  Modules/_csv.o  Modules/_ctypes/_ctypes.o Modules/_ctypes/callbacks.o Modules/_ctypes/callproc.o Modules/_ctypes/stgdict.o Modules/_ctypes/cfield.o  Modules/_cursesmodule.o  Modules/_curses_panel.o  Modules/_datetimemodule.o  Modules/_decimal/_decimal.o  Modules/_elementtree.o  Modules/_hashopenssl.o  Modules/_heapqmodule.o  Modules/_interpchannelsmodule.o  Modules/_interpqueuesmodule.o  Modules/_interpretersmodule.o  Modules/_json.o  Modules/_lsprof.o Modules/rotatingtree.o  Modules/_lzmamodule.o  Modules/md5module.o Modules/_hacl/Hacl_Hash_MD5.o  Modules/cjkcodecs/multibytecodec.o  Modules/_multiprocessing/multiprocessing.o Modules/_multiprocessing/semaphore.o  Modules/_opcode.o  Modules/_pickle.o  Modules/_multiprocessing/posixshmem.o  Modules/_posixsubprocess.o  Modules/_queuemodule.o  Modules/_randommodule.o  Modules/sha1module.o Modules/_hacl/Hacl_Hash_SHA1.o  Modules/sha2module.o Modules/_hacl/Hacl_Hash_SHA2.o  Modules/sha3module.o Modules/_hacl/Hacl_Hash_SHA3.o  Modules/socketmodule.o  Modules/_sqlite/connection.o Modules/_sqlite/cursor.o Modules/_sqlite/microprotocols.o Modules/_sqlite/module.o Modules/_sqlite/prepare_protocol.o Modules/_sqlite/row.o Modules/_sqlite/statement.o Modules/_sqlite/util.o Modules/_sqlite/blob.o  Modules/_ssl.o  Modules/_statisticsmodule.o  Modules/_struct.o  Modules/_suggestions.o  Modules/_sysconfig.o  Modules/_testinternalcapi.o Modules/_testinternalcapi/pytime.o Modules/_testinternalcapi/set.o Modules/_testinternalcapi/test_critical_sections.o Modules/_testinternalcapi/test_lock.o  Modules/_typingmodule.o  Modules/_uuidmodule.o  Modules/_xxtestfuzz/_xxtestfuzz.o Modules/_xxtestfuzz/fuzzer.o  Modules/_zoneinfo.o  Modules/arraymodule.o  Modules/binascii.o  Modules/cmathmodule.o  Modules/fcntlmodule.o  Modules/grpmodule.o  Modules/mathmodule.o  Modules/mmapmodule.o  Modules/pyexpat.o  Modules/readline.o  Modules/resource.o  Modules/selectmodule.o  Modules/syslogmodule.o  Modules/termios.o  Modules/unicodedata.o  Modules/xxsubtype.o  Modules/zlibmodule.o  Modules/atexitmodule.o  Modules/faulthandler.o  Modules/posixmodule.o  Modules/signalmodule.o  Modules/_tracemalloc.o  Modules/_codecsmodule.o  Modules/_collectionsmodule.o  Modules/errnomodule.o  Modules/_io/_iomodule.o Modules/_io/iobase.o Modules/_io/fileio.o Modules/_io/bytesio.o Modules/_io/bufferedio.o Modules/_io/textio.o Modules/_io/stringio.o  Modules/itertoolsmodule.o  Modules/_sre/sre.o  Modules/_threadmodule.o  Modules/timemodule.o  Modules/_weakref.o  Modules/_abc.o  Modules/_functoolsmodule.o  Modules/_localemodule.o  Modules/_operator.o  Modules/_stat.o  Modules/symtablemodule.o  Modules/pwdmodule.o",
    "MODSHARED_NAMES": "_ctypes_test _dbm _testbuffer _testcapi _testexternalinspection _testimportmultiple _testlimitedcapi _testmultiphase _testsinglephase _tkinter",
    "MODULE_ARRAY_LDFLAGS": "",
    "MODULE_ARRAY_STATE": "disabled",
    "MODULE_ATEXIT_LDFLAGS": "",
    "MODULE_BINASCII_LDFLAGS": "",
    "MODULE_BINASCII_STATE": "disabled",
    "MODULE_CMATH_DEPS": "./Modules/_math.h",
    "MODULE_CMATH_STATE": "disabled",
    "MODULE_DEPS_SHARED": "Modules/config.c",
    "MODULE_DEPS_STATIC": "Modules/config.c",
    "MODULE_ERRNO_LDFLAGS": "",
    "MODULE_FAULTHANDLER_LDFLAGS": "",
    "MODULE_FCNTL_LDFLAGS": "",
    "MODULE_FCNTL_STATE": "disabled",
    "MODULE_GRP_LDFLAGS": "",
    "MODULE_GRP_STATE": "disabled",
    "MODULE_ITERTOOLS_LDFLAGS": "",
    "MODULE_MATH_DEPS": "./Modules/_math.h",
    "MODULE_MATH_STATE": "disabled",
    "MODULE_MMAP_LDFLAGS": "",
    "MODULE_MMAP_STATE": "disabled",
    "MODULE_OBJS": "\\",
    "MODULE_POSIX_LDFLAGS": "",
    "MODULE_PWD_LDFLAGS": "",
    "MODULE_PWD_STATE": "disabled",
    "MODULE_PYEXPAT_DEPS": "",
    "MODULE_PYEXPAT_STATE": "disabled",
    "MODULE_READLINE_STATE": "disabled",
    "MODULE_RESOURCE_LDFLAGS": "",
    "MODULE_RESOURCE_STATE": "disabled",
    "MODULE_SELECT_LDFLAGS": "",
    "MODULE_SELECT_STATE": "disabled",
    "MODULE_SYSLOG_LDFLAGS": "",
    "MODULE_SYSLOG_STATE": "disabled",
    "MODULE_TERMIOS_LDFLAGS": "",
    "MODULE_TERMIOS_STATE": "disabled",
    "MODULE_TIME_LDFLAGS": "",
    "MODULE_TIME_STATE": "disabled",
    "MODULE_UNICODEDATA_DEPS": "./Modules/unicodedata_db.h ./Modules/unicodename_db.h",
    "MODULE_UNICODEDATA_LDFLAGS": "",
    "MODULE_UNICODEDATA_STATE": "disabled",
    "MODULE_XXLIMITED_35_STATE": "disabled",
    "MODULE_XXLIMITED_STATE": "disabled",
    "MODULE_XXSUBTYPE_LDFLAGS": "",
    "MODULE_XXSUBTYPE_STATE": "disabled",
    "MODULE_ZLIB_STATE": "disabled",
    "MODULE__ABC_LDFLAGS": "",
    "MODULE__ASYNCIO_LDFLAGS": "",
    "MODULE__ASYNCIO_STATE": "disabled",
    "MODULE__BISECT_LDFLAGS": "",
    "MODULE__BISECT_STATE": "disabled",
    "MODULE__BLAKE2_DEPS": "./Modules/_blake2/impl/blake2-config.h ./Modules/_blake2/impl/blake2-impl.h ./Modules/_blake2/impl/blake2.h ./Modules/_blake2/impl/blake2b-load-sse2.h ./Modules/_blake2/impl/blake2b-load-sse41.h ./Modules/_blake2/impl/blake2b-ref.c ./Modules/_blake2/impl/blake2b-round.h ./Modules/_blake2/impl/blake2b.c ./Modules/_blake2/impl/blake2s-load-sse2.h ./Modules/_blake2/impl/blake2s-load-sse41.h ./Modules/_blake2/impl/blake2s-load-xop.h ./Modules/_blake2/impl/blake2s-ref.c ./Modules/_blake2/impl/blake2s-round.h ./Modules/_blake2/impl/blake2s.c ./Modules/_blake2/blake2module.h ./Modules/hashlib.h",
    "MODULE__BLAKE2_LDFLAGS": "",
    "MODULE__BLAKE2_STATE": "disabled",
    "MODULE__BZ2_STATE": "disabled",
    "MODULE__CODECS_CN_DEPS": "./Modules/cjkcodecs/mappings_cn.h ./Modules/cjkcodecs/multibytecodec.h ./Modules/cjkcodecs/cjkcodecs.h",
    "MODULE__CODECS_CN_LDFLAGS": "",
    "MODULE__CODECS_CN_STATE": "disabled",
    "MODULE__CODECS_HK_DEPS": "./Modules/cjkcodecs/mappings_hk.h  ./Modules/cjkcodecs/multibytecodec.h ./Modules/cjkcodecs/cjkcodecs.h",
    "MODULE__CODECS_HK_LDFLAGS": "",
    "MODULE__CODECS_HK_STATE": "disabled",
    "MODULE__CODECS_ISO2022_DEPS": "./Modules/cjkcodecs/mappings_jisx0213_pair.h ./Modules/cjkcodecs/alg_jisx0201.h ./Modules/cjkcodecs/emu_jisx0213_2000.h ./Modules/cjkcodecs/multibytecodec.h ./Modules/cjkcodecs/cjkcodecs.h",
    "MODULE__CODECS_ISO2022_LDFLAGS": "",
    "MODULE__CODECS_ISO2022_STATE": "disabled",
    "MODULE__CODECS_JP_DEPS": "./Modules/cjkcodecs/mappings_jisx0213_pair.h ./Modules/cjkcodecs/alg_jisx0201.h ./Modules/cjkcodecs/emu_jisx0213_2000.h ./Modules/cjkcodecs/mappings_jp.h ./Modules/cjkcodecs/multibytecodec.h ./Modules/cjkcodecs/cjkcodecs.h",
    "MODULE__CODECS_JP_LDFLAGS": "",
    "MODULE__CODECS_JP_STATE": "disabled",
    "MODULE__CODECS_KR_DEPS": "./Modules/cjkcodecs/mappings_kr.h ./Modules/cjkcodecs/multibytecodec.h ./Modules/cjkcodecs/cjkcodecs.h",
    "MODULE__CODECS_KR_LDFLAGS": "",
    "MODULE__CODECS_KR_STATE": "disabled",
    "MODULE__CODECS_LDFLAGS": "",
    "MODULE__CODECS_TW_DEPS": "./Modules/cjkcodecs/mappings_tw.h ./Modules/cjkcodecs/multibytecodec.h ./Modules/cjkcodecs/cjkcodecs.h",
    "MODULE__CODECS_TW_LDFLAGS": "",
    "MODULE__CODECS_TW_STATE": "disabled",
    "MODULE__COLLECTIONS_LDFLAGS": "",
    "MODULE__CONTEXTVARS_LDFLAGS": "",
    "MODULE__CONTEXTVARS_STATE": "disabled",
    "MODULE__CSV_LDFLAGS": "",
    "MODULE__CSV_STATE": "disabled",
    "MODULE__CTYPES_DEPS": "./Modules/_ctypes/ctypes.h",
    "MODULE__CTYPES_MALLOC_CLOSURE": "",
    "MODULE__CTYPES_STATE": "disabled",
    "MODULE__CTYPES_TEST_STATE": "disabled",
    "MODULE__CURSES_DEPS": "./Include/py_curses.h",
    "MODULE__CURSES_PANEL_DEPS": "./Include/py_curses.h",
    "MODULE__CURSES_PANEL_STATE": "disabled",
    "MODULE__CURSES_STATE": "disabled",
    "MODULE__DATETIME_DEPS": "./Include/datetime.h",
    "MODULE__DATETIME_LDFLAGS": "",
    "MODULE__DATETIME_STATE": "disabled",
    "MODULE__DBM_STATE": "disabled",
    "MODULE__DECIMAL_DEPS": "./Modules/_decimal/docstrings.h",
    "MODULE__DECIMAL_STATE": "disabled",
    "MODULE__ELEMENTTREE_DEPS": "./Modules/pyexpat.c",
    "MODULE__ELEMENTTREE_STATE": "disabled",
    "MODULE__FUNCTOOLS_LDFLAGS": "",
    "MODULE__GDBM_STATE": "disabled",
    "MODULE__HASHLIB_DEPS": "./Modules/hashlib.h",
    "MODULE__HASHLIB_STATE": "disabled",
    "MODULE__HEAPQ_LDFLAGS": "",
    "MODULE__HEAPQ_STATE": "disabled",
    "MODULE__INTERPCHANNELS_LDFLAGS": "",
    "MODULE__INTERPCHANNELS_STATE": "disabled",
    "MODULE__INTERPQUEUES_LDFLAGS": "",
    "MODULE__INTERPQUEUES_STATE": "disabled",
    "MODULE__INTERPRETERS_LDFLAGS": "",
    "MODULE__INTERPRETERS_STATE": "disabled",
    "MODULE__IO_DEPS": "./Modules/_io/_iomodule.h",
    "MODULE__IO_LDFLAGS": "",
    "MODULE__IO_STATE": "disabled",
    "MODULE__JSON_LDFLAGS": "",
    "MODULE__JSON_STATE": "disabled",
    "MODULE__LOCALE_LDFLAGS": "",
    "MODULE__LSPROF_LDFLAGS": "",
    "MODULE__LSPROF_STATE": "disabled",
    "MODULE__LZMA_STATE": "disabled",
    "MODULE__MD5_DEPS": "./Modules/hashlib.h \\ Modules/_hacl/Hacl_Hash_MD5.h Modules/_hacl/Hacl_Hash_MD5.c",
    "MODULE__MD5_STATE": "disabled",
    "MODULE__MULTIBYTECODEC_DEPS": "./Modules/cjkcodecs/multibytecodec.h",
    "MODULE__MULTIBYTECODEC_LDFLAGS": "",
    "MODULE__MULTIBYTECODEC_STATE": "disabled",
    "MODULE__MULTIPROCESSING_LDFLAGS": "",
    "MODULE__MULTIPROCESSING_STATE": "disabled",
    "MODULE__OPCODE_LDFLAGS": "",
    "MODULE__OPCODE_STATE": "disabled",
    "MODULE__OPERATOR_LDFLAGS": "",
    "MODULE__PICKLE_LDFLAGS": "",
    "MODULE__PICKLE_STATE": "disabled",
    "MODULE__POSIXSHMEM_STATE": "disabled",
    "MODULE__POSIXSUBPROCESS_LDFLAGS": "",
    "MODULE__POSIXSUBPROCESS_STATE": "disabled",
    "MODULE__QUEUE_LDFLAGS": "",
    "MODULE__QUEUE_STATE": "disabled",
    "MODULE__RANDOM_LDFLAGS": "",
    "MODULE__RANDOM_STATE": "disabled",
    "MODULE__SCPROXY_STATE": "n/a",
    "MODULE__SHA1_DEPS": "./Modules/hashlib.h \\ Modules/_hacl/Hacl_Hash_SHA1.h Modules/_hacl/Hacl_Hash_SHA1.c",
    "MODULE__SHA1_STATE": "disabled",
    "MODULE__SHA2_DEPS": "./Modules/hashlib.h \\ Modules/_hacl/libHacl_Hash_SHA2.a",
    "MODULE__SHA2_STATE": "disabled",
    "MODULE__SHA3_DEPS": "./Modules/hashlib.h \\ Modules/_hacl/Hacl_Hash_SHA3.h Modules/_hacl/Hacl_Hash_SHA3.c",
    "MODULE__SHA3_STATE": "disabled",
    "MODULE__SIGNAL_LDFLAGS": "",
    "MODULE__SOCKET_DEPS": "./Modules/socketmodule.h ./Modules/addrinfo.h ./Modules/getaddrinfo.c ./Modules/getnameinfo.c",
    "MODULE__SOCKET_LDFLAGS": "",
    "MODULE__SOCKET_STATE": "disabled",
    "MODULE__SQLITE3_DEPS": "./Modules/_sqlite/connection.h ./Modules/_sqlite/cursor.h ./Modules/_sqlite/microprotocols.h ./Modules/_sqlite/module.h ./Modules/_sqlite/prepare_protocol.h ./Modules/_sqlite/row.h ./Modules/_sqlite/util.h",
    "MODULE__SQLITE3_STATE": "disabled",
    "MODULE__SRE_LDFLAGS": "",
    "MODULE__SSL_DEPS": "./Modules/_ssl.h ./Modules/_ssl/cert.c ./Modules/_ssl/debughelpers.c ./Modules/_ssl/misc.c ./Modules/_ssl_data_111.h ./Modules/_ssl_data_300.h ./Modules/socketmodule.h",
    "MODULE__SSL_STATE": "disabled",
    "MODULE__STATISTICS_LDFLAGS": "",
    "MODULE__STATISTICS_STATE": "disabled",
    "MODULE__STAT_LDFLAGS": "",
    "MODULE__STRUCT_LDFLAGS": "",
    "MODULE__STRUCT_STATE": "disabled",
    "MODULE__SUGGESTIONS_LDFLAGS": "",
    "MODULE__SYMTABLE_LDFLAGS": "",
    "MODULE__SYSCONFIG_LDFLAGS": "",
    "MODULE__TESTBUFFER_STATE": "disabled",
    "MODULE__TESTCAPI_DEPS": "./Modules/_testcapi/parts.h ./Modules/_testcapi/util.h",
    "MODULE__TESTCAPI_STATE": "disabled",
    "MODULE__TESTCLINIC_LIMITED_STATE": "disabled",
    "MODULE__TESTCLINIC_STATE": "disabled",
    "MODULE__TESTEXTERNALINSPECTION_STATE": "disabled",
    "MODULE__TESTIMPORTMULTIPLE_STATE": "disabled",
    "MODULE__TESTINTERNALCAPI_DEPS": "./Modules/_testinternalcapi/parts.h",
    "MODULE__TESTINTERNALCAPI_STATE": "disabled",
    "MODULE__TESTLIMITEDCAPI_DEPS": "./Modules/_testlimitedcapi/testcapi_long.h ./Modules/_testlimitedcapi/parts.h ./Modules/_testlimitedcapi/util.h",
    "MODULE__TESTLIMITEDCAPI_STATE": "disabled",
    "MODULE__TESTMULTIPHASE_STATE": "disabled",
    "MODULE__TESTSINGLEPHASE_STATE": "disabled",
    "MODULE__THREAD_LDFLAGS": "",
    "MODULE__TKINTER_STATE": "disabled",
    "MODULE__TRACEMALLOC_LDFLAGS": "",
    "MODULE__TYPING_LDFLAGS": "",
    "MODULE__TYPING_STATE": "disabled",
    "MODULE__UUID_STATE": "disabled",
    "MODULE__WEAKREF_LDFLAGS": "",
    "MODULE__XXTESTFUZZ_LDFLAGS": "",
    "MODULE__XXTESTFUZZ_STATE": "disabled",
    "MODULE__ZONEINFO_LDFLAGS": "",
    "MODULE__ZONEINFO_STATE": "disabled",
    "MULTIARCH": "x86_64-linux-gnu",
    "MULTIARCH_CPPFLAGS": "-DMULTIARCH=\\\"x86_64-linux-gnu\\\"",
    "MVWDELCH_IS_EXPRESSION": 1,
    "NO_AS_NEEDED": "-Wl,--no-as-needed",
    "OBJECT_OBJS": "\\",
    "OPT": "-DNDEBUG -g -O3 -Wall",
    "PACKAGE_BUGREPORT": 0,
    "PACKAGE_NAME": 0,
    "PACKAGE_STRING": 0,
    "PACKAGE_TARNAME": 0,
    "PACKAGE_URL": 0,
    "PACKAGE_VERSION": 0,
    "PARSER_HEADERS": "\\",
    "PARSER_OBJS": "\\ \\ \\ Parser/myreadline.o",
    "PEGEN_HEADERS": "\\",
    "PEGEN_OBJS": "\\",
    "PGO_PROF_GEN_FLAG": "-fprofile-instr-generate",
    "PGO_PROF_USE_FLAG": "-fprofile-instr-use=\"$(shell pwd)/code.profclangd\"",
    "PLATLIBDIR": "lib",
    "POBJS": "\\",
    "POSIX_SEMAPHORES_NOT_ENABLED": 0,
    "PROFILE_TASK": "-m test --pgo -j 4 --ignore test.test_bytes.BytesTest.test_from_format",
    "PTHREAD_KEY_T_IS_COMPATIBLE_WITH_INT": 1,
    "PTHREAD_SYSTEM_SCHED_SUPPORTED": 1,
    "PURIFY": "",
    "PY3LIBRARY": "libpython3.so",
    "PYLONG_BITS_IN_DIGIT": 0,
    "PYTHON": "python",
    "PYTHONFRAMEWORK": "",
    "PYTHONFRAMEWORKDIR": "no-framework",
    "PYTHONFRAMEWORKINSTALLDIR": "",
    "PYTHONFRAMEWORKINSTALLNAMEPREFIX": "",
    "PYTHONFRAMEWORKPREFIX": "",
    "PYTHONPATH": "",
    "PYTHON_FOR_BUILD": "_PYTHON_PROJECT_BASE=/build/Python-3.13.14 _PYTHON_HOST_PLATFORM=$(_PYTHON_HOST_PLATFORM) PYTHONPATH=./Lib _PYTHON_SYSCONFIGDATA_NAME=_sysconfigdata__linux_x86_64-linux-gnu _PYTHON_SYSCONFIGDATA_PATH=$(shell test -f pybuilddir.txt && echo /build/Python-3.13.14/`cat pybuilddir.txt`) /tools/host/bin/python3.13",
    "PYTHON_FOR_BUILD_DEPS": "",
    "PYTHON_FOR_FREEZE": "/tools/host/bin/python3.13",
    "PYTHON_FOR_REGEN": "",
    "PYTHON_HEADERS": "\\",
    "PYTHON_OBJS": "\\",
    "PY_BUILTIN_HASHLIB_HASHES": "\"md5,sha1,sha2,sha3,blake2\"",
    "PY_BUILTIN_MODULE_CFLAGS": "-fno-strict-overflow -Wsign-compare -Wunreachable-code -DNDEBUG -g -O3 -Wall -O3  -fno-omit-frame-pointer -mno-omit-leaf-frame-pointer  -fPIC   -D_Py_TIER2=3 -D_Py_JIT -flto=thin -g -fno-pie -std=c11 -Wextra -Wno-unused-parameter -Wno-missing-field-initializers -Wstrict-prototypes -Werror=implicit-function-declaration -fvisibility=hidden -fprofile-instr-use=/build/Python-3.13.14/code.profclangd -I./Include/internal -I./Include/internal/mimalloc -I. -I./Include -O3  -fno-omit-frame-pointer -mno-omit-leaf-frame-pointer  -fPIC   -fPIC -DPy_BUILD_CORE_BUILTIN",
    "PY_CFLAGS": "-fno-strict-overflow -Wsign-compare -Wunreachable-code -DNDEBUG -g -O3 -Wall -O3  -fno-omit-frame-pointer -mno-omit-leaf-frame-pointer  -fPIC  ",
    "PY_CFLAGS_NODIST": "-D_Py_TIER2=3 -D_Py_JIT -flto=thin -g -fno-pie -std=c11 -Wextra -Wno-unused-parameter -Wno-missing-field-initializers -Wstrict-prototypes -Werror=implicit-function-declaration -fvisibility=hidden -fprofile-instr-use=/build/Python-3.13.14/code.profclangd -I./Include/internal -I./Include/internal/mimalloc",
    "PY_COERCE_C_LOCALE": 1,
    "PY_CORE_CFLAGS": "-fno-strict-overflow -Wsign-compare -Wunreachable-code -DNDEBUG -g -O3 -Wall -O3  -fno-omit-frame-pointer -mno-omit-leaf-frame-pointer  -fPIC   -D_Py_TIER2=3 -D_Py_JIT -flto=thin -g -fno-pie -std=c11 -Wextra -Wno-unused-parameter -Wno-missing-field-initializers -Wstrict-prototypes -Werror=implicit-function-declaration -fvisibility=hidden -fprofile-instr-use=/build/Python-3.13.14/code.profclangd -I./Include/internal -I./Include/internal/mimalloc -I. -I./Include -O3  -fno-omit-frame-pointer -mno-omit-leaf-frame-pointer  -fPIC   -fPIC -DPy_BUILD_CORE",
    "PY_CORE_LDFLAGS": "-Wl,-z,noexecstack  -Wl,--exclude-libs,ALL  -flto=thin -g -Wl,--emit-relocs",
    "PY_CPPFLAGS": "-I. -I./Include -O3  -fno-omit-frame-pointer -mno-omit-leaf-frame-pointer  -fPIC  ",
    "PY_ENABLE_SHARED": 1,
    "PY_HAVE_PERF_TRAMPOLINE": 1,
    "PY_LDFLAGS": "-Wl,-z,noexecstack  -Wl,--exclude-libs,ALL ",
    "PY_LDFLAGS_NODIST": "-flto=thin -g -Wl,--emit-relocs",
    "PY_LDFLAGS_NOLTO": "-Wl,-z,noexecstack  -Wl,--exclude-libs,ALL  -flto=thin",
    "PY_SQLITE_ENABLE_LOAD_EXTENSION": 0,
    "PY_SQLITE_HAVE_SERIALIZE": 1,
    "PY_SSL_DEFAULT_CIPHERS": 1,
    "PY_SSL_DEFAULT_CIPHER_STRING": 0,
    "PY_STDMODULE_CFLAGS": "-fno-strict-overflow -Wsign-compare -Wunreachable-code -DNDEBUG -g -O3 -Wall -O3  -fno-omit-frame-pointer -mno-omit-leaf-frame-pointer  -fPIC   -D_Py_TIER2=3 -D_Py_JIT -flto=thin -g -fno-pie -std=c11 -Wextra -Wno-unused-parameter -Wno-missing-field-initializers -Wstrict-prototypes -Werror=implicit-function-declaration -fvisibility=hidden -fprofile-instr-use=/build/Python-3.13.14/code.profclangd -I./Include/internal -I./Include/internal/mimalloc -I. -I./Include -O3  -fno-omit-frame-pointer -mno-omit-leaf-frame-pointer  -fPIC   -fPIC",
    "PY_SUPPORT_TIER": 2,
    "Py_DEBUG": 0,
    "Py_ENABLE_SHARED": 1,
    "Py_GIL_DISABLED": 0,
    "Py_HASH_ALGORITHM": 0,
    "Py_RL_STARTUP_HOOK_TAKES_ARGS": 0,
    "Py_STATS": 0,
    "Py_SUNOS_VERSION": 0,
    "Py_TRACE_REFS": 0,
    "QUICKTESTOPTS": "-x test_subprocess test_io \\",
    "READELF": "@READELF@",
    "RESSRCDIR": "",
    "RETSIGTYPE": "void",
    "RUNSHARED": "LD_LIBRARY_PATH=/build/Python-3.13.14",
    "SCRIPTDIR": "/install/lib",
    "SCRIPT_IDLE": "build/scripts-3.13/idle3.13",
    "SCRIPT_PYDOC": "build/scripts-3.13/pydoc3.13",
    "SETPGRP_HAVE_ARG": 0,
    "SHAREDMODS": "Modules/_ctypes_test.cpython-313-x86_64-linux-gnu.so Modules/_dbm.cpython-313-x86_64-linux-gnu.so Modules/_testbuffer.cpython-313-x86_64-linux-gnu.so Modules/_testcapi.cpython-313-x86_64-linux-gnu.so Modules/_testexternalinspection.cpython-313-x86_64-linux-gnu.so Modules/_testimportmultiple.cpython-313-x86_64-linux-gnu.so Modules/_testlimitedcapi.cpython-313-x86_64-linux-gnu.so Modules/_testmultiphase.cpython-313-x86_64-linux-gnu.so Modules/_testsinglephase.cpython-313-x86_64-linux-gnu.so Modules/_tkinter.cpython-313-x86_64-linux-gnu.so",
    "SHELL": "/bin/sh -e",
    "SHLIBS": "-lpthread -ldl  -lutil",
    "SHLIB_SUFFIX": ".so",
    "SIGNED_RIGHT_SHIFT_ZERO_FILLS": 0,
    "SITEPATH": "",
    "SIZEOF_DOUBLE": 8,
    "SIZEOF_FLOAT": 4,
    "SIZEOF_FPOS_T": 16,
    "SIZEOF_INT": 4,
    "SIZEOF_LONG": 8,
    "SIZEOF_LONG_DOUBLE": 16,
    "SIZEOF_LONG_LONG": 8,
    "SIZEOF_OFF_T": 8,
    "SIZEOF_PID_T": 4,
    "SIZEOF_PTHREAD_KEY_T": 4,
    "SIZEOF_PTHREAD_T": 8,
    "SIZEOF_SHORT": 2,
    "SIZEOF_SIZE_T": 8,
    "SIZEOF_TIME_T": 8,
    "SIZEOF_UINTPTR_T": 8,
    "SIZEOF_VOID_P": 8,
    "SIZEOF_WCHAR_T": 4,
    "SIZEOF__BOOL": 1,
    "SOABI": "cpython-313-x86_64-linux-gnu",
    "SRCDIRS": "Modules   Modules/_blake2   Modules/_ctypes   Modules/_decimal   Modules/_decimal/libmpdec   Modules/_hacl   Modules/_io   Modules/_multiprocessing   Modules/_sqlite   Modules/_sre   Modules/_testcapi   Modules/_testinternalcapi   Modules/_testlimitedcapi   Modules/_xxtestfuzz   Modules/cjkcodecs   Modules/expat   Objects   Objects/mimalloc   Objects/mimalloc/prim   Parser   Parser/tokenizer   Parser/lexer   Programs   Python   Python/frozen_modules",
    "SRC_GDB_HOOKS": "./Tools/gdb/libpython.py",
    "STATIC_LIBPYTHON": 1,
    "STDC_HEADERS": 1,
    "STRICT_SYSV_CURSES": "/* Don't use ncurses extensions */",
    "STRIPFLAG": "-s",
    "SUBDIRS": "",
    "SUBDIRSTOO": "Include Lib Misc",
    "SYSLIBS": "-lm",
    "SYS_SELECT_WITH_SYS_TIME": 1,
    "TESTOPTS": "",
    "TESTPATH": "",
    "TESTPYTHON": "LD_LIBRARY_PATH=/build/Python-3.13.14 _PYTHON_PROJECT_BASE=/build/Python-3.13.14 _PYTHON_HOST_PLATFORM=$(_PYTHON_HOST_PLATFORM) PYTHONPATH=./Lib _PYTHON_SYSCONFIGDATA_NAME=_sysconfigdata__linux_x86_64-linux-gnu _PYTHON_SYSCONFIGDATA_PATH=$(shell test -f pybuilddir.txt && echo /build/Python-3.13.14/`cat pybuilddir.txt`) /tools/host/bin/python3.13",
    "TESTPYTHONOPTS": "",
    "TESTRUNNER": "LD_LIBRARY_PATH=/build/Python-3.13.14 _PYTHON_PROJECT_BASE=/build/Python-3.13.14 _PYTHON_HOST_PLATFORM=$(_PYTHON_HOST_PLATFORM) PYTHONPATH=./Lib _PYTHON_SYSCONFIGDATA_NAME=_sysconfigdata__linux_x86_64-linux-gnu _PYTHON_SYSCONFIGDATA_PATH=$(shell test -f pybuilddir.txt && echo /build/Python-3.13.14/`cat pybuilddir.txt`) /tools/host/bin/python3.13 -m test",
    "TESTSUBDIRS": "idlelib/idle_test \\",
    "TESTTIMEOUT": "",
    "TEST_MODULES": "yes",
    "THREAD_STACK_SIZE": 0,
    "TIMEMODULE_LIB": 0,
    "TM_IN_SYS_TIME": 0,
    "TOKENIZER_HEADERS": "\\",
    "TOKENIZER_OBJS": "\\",
    "TZPATH": "/usr/share/zoneinfo:/usr/lib/zoneinfo:/usr/share/lib/zoneinfo:/etc/zoneinfo",
    "UNICODE_DEPS": "\\",
    "UNIVERSALSDK": "",
    "UPDATE_FILE": "./Tools/build/update_file.py",
    "USE_COMPUTED_GOTOS": 0,
    "VERSION": "3.13",
    "WASM_ASSETS_DIR": "./install",
    "WASM_STDLIB": "./install/lib/python3.13/os.py",
    "WHEEL_PKG_DIR": "",
    "WINDOW_HAS_FLAGS": 0,
    "WITH_DECIMAL_CONTEXTVAR": 1,
    "WITH_DOC_STRINGS": 1,
    "WITH_DTRACE": 0,
    "WITH_DYLD": 0,
    "WITH_EDITLINE": 1,
    "WITH_FREELISTS": 1,
    "WITH_LIBINTL": 0,
    "WITH_MIMALLOC": 1,
    "WITH_NEXT_FRAMEWORK": 0,
    "WITH_PYMALLOC": 1,
    "WITH_VALGRIND": 0,
    "X87_DOUBLE_ROUNDING": 0,
    "XMLLIBSUBDIRS": "xml xml/dom xml/etree xml/parsers xml/sax",
    "abs_builddir": "/build/Python-3.13.14",
    "abs_srcdir": "/build/Python-3.13.14",
    "datarootdir": "/install/share",
    "exec_prefix": "/install",
    "prefix": "/install",
    "srcdir": "."
}


# Copyright (C) 2001-2006 Python Software Foundation
# Author: Ben Gertzfield
# Contact: email-sig@python.org

"""Quoted-printable content transfer encoding per RFCs 2045-2047.

This module handles the content transfer encoding method defined in RFC 2045
to encode US ASCII-like 8-bit data called `quoted-printable'.  It is used to
safely encode text that is in a character set similar to the 7-bit US ASCII
character set, but that includes some 8-bit characters that are normally not
allowed in email bodies or headers.

Quoted-printable is very space-inefficient for encoding binary files; use the
email.base64mime module for that instead.

This module provides an interface to encode and decode both headers and bodies
with quoted-printable encoding.

RFC 2045 defines a method for including character set information in an
`encoded-word' in a header.  This method is commonly used for 8-bit real names
in To:/From:/Cc: etc. fields, as well as Subject: lines.

This module does not do the line wrapping or end-of-line character
conversion necessary for proper internationalized headers; it only
does dumb encoding and decoding.  To deal with the various line
wrapping issues, use the email.header module.
"""

__all__ = [
    'body_decode',
    'body_encode',
    'body_length',
    'decode',
    'decodestring',
    'header_decode',
    'header_encode',
    'header_length',
    'quote',
    'unquote',
    ]

import re

from string import ascii_letters, digits, hexdigits

CRLF = '\r\n'
NL = '\n'
EMPTYSTRING = ''

# Build a mapping of octets to the expansion of that octet.  Since we're only
# going to have 256 of these things, this isn't terribly inefficient
# space-wise.  Remember that headers and bodies have different sets of safe
# characters.  Initialize both maps with the full expansion, and then override
# the safe bytes with the more compact form.
_QUOPRI_MAP = ['=%02X' % c for c in range(256)]
_QUOPRI_HEADER_MAP = _QUOPRI_MAP[:]
_QUOPRI_BODY_MAP = _QUOPRI_MAP[:]

# Safe header bytes which need no encoding.
for c in b'-!*+/' + ascii_letters.encode('ascii') + digits.encode('ascii'):
    _QUOPRI_HEADER_MAP[c] = chr(c)
# Headers have one other special encoding; spaces become underscores.
_QUOPRI_HEADER_MAP[ord(' ')] = '_'

# Safe body bytes which need no encoding.
for c in (b' !"#$%&\'()*+,-./0123456789:;<>'
          b'?@ABCDEFGHIJKLMNOPQRSTUVWXYZ[\\]^_`'
          b'abcdefghijklmnopqrstuvwxyz{|}~\t'):
    _QUOPRI_BODY_MAP[c] = chr(c)



# Helpers
def header_check(octet):
    """Return True if the octet should be escaped with header quopri."""
    return chr(octet) != _QUOPRI_HEADER_MAP[octet]


def body_check(octet):
    """Return True if the octet should be escaped with body quopri."""
    return chr(octet) != _QUOPRI_BODY_MAP[octet]


def header_length(bytearray):
    """Return a header quoted-printable encoding length.

    Note that this does not include any RFC 2047 chrome added by
    `header_encode()`.

    :param bytearray: An array of bytes (a.k.a. octets).
    :return: The length in bytes of the byte array when it is encoded with
        quoted-printable for headers.
    """
    return sum(len(_QUOPRI_HEADER_MAP[octet]) for octet in bytearray)


def body_length(bytearray):
    """Return a body quoted-printable encoding length.

    :param bytearray: An array of bytes (a.k.a. octets).
    :return: The length in bytes of the byte array when it is encoded with
        quoted-printable for bodies.
    """
    return sum(len(_QUOPRI_BODY_MAP[octet]) for octet in bytearray)


def _max_append(L, s, maxlen, extra=''):
    if not isinstance(s, str):
        s = chr(s)
    if not L:
        L.append(s.lstrip())
    elif len(L[-1]) + len(s) <= maxlen:
        L[-1] += extra + s
    else:
        L.append(s.lstrip())


def unquote(s):
    """Turn a string in the form =AB to the ASCII character with value 0xab"""
    return chr(int(s[1:3], 16))


def quote(c):
    return _QUOPRI_MAP[ord(c)]


def header_encode(header_bytes, charset='iso-8859-1'):
    """Encode a single header line with quoted-printable (like) encoding.

    Defined in RFC 2045, this `Q' encoding is similar to quoted-printable, but
    used specifically for email header fields to allow charsets with mostly 7
    bit characters (and some 8 bit) to remain more or less readable in non-RFC
    2045 aware mail clients.

    charset names the character set to use in the RFC 2046 header.  It
    defaults to iso-8859-1.
    """
    # Return empty headers as an empty string.
    if not header_bytes:
        return ''
    # Iterate over every byte, encoding if necessary.
    encoded = header_bytes.decode('latin1').translate(_QUOPRI_HEADER_MAP)
    # Now add the RFC chrome to each encoded chunk and glue the chunks
    # together.
    return '=?%s?q?%s?=' % (charset, encoded)


_QUOPRI_BODY_ENCODE_MAP = _QUOPRI_BODY_MAP[:]
for c in b'\r\n':
    _QUOPRI_BODY_ENCODE_MAP[c] = chr(c)
del c

def body_encode(body, maxlinelen=76, eol=NL):
    """Encode with quoted-printable, wrapping at maxlinelen characters.

    Each line of encoded text will end with eol, which defaults to "\\n".  Set
    this to "\\r\\n" if you will be using the result of this function directly
    in an email.

    Each line will be wrapped at, at most, maxlinelen characters before the
    eol string (maxlinelen defaults to 76 characters, the maximum value
    permitted by RFC 2045).  Long lines will have the 'soft line break'
    quoted-printable character "=" appended to them, so the decoded text will
    be identical to the original text.

    The minimum maxlinelen is 4 to have room for a quoted character ("=XX")
    followed by a soft line break.  Smaller values will generate a
    ValueError.

    """

    if maxlinelen < 4:
        raise ValueError("maxlinelen must be at least 4")
    if not body:
        return body

    # quote special characters
    body = body.translate(_QUOPRI_BODY_ENCODE_MAP)

    soft_break = '=' + eol
    # leave space for the '=' at the end of a line
    maxlinelen1 = maxlinelen - 1

    encoded_body = []
    append = encoded_body.append

    for line in body.splitlines():
        # break up the line into pieces no longer than maxlinelen - 1
        start = 0
        laststart = len(line) - 1 - maxlinelen
        while start <= laststart:
            stop = start + maxlinelen1
            # make sure we don't break up an escape sequence
            if line[stop - 2] == '=':
                append(line[start:stop - 1])
                start = stop - 2
            elif line[stop - 1] == '=':
                append(line[start:stop])
                start = stop - 1
            else:
                append(line[start:stop] + '=')
                start = stop

        # handle rest of line, special case if line ends in whitespace
        if line and line[-1] in ' \t':
            room = start - laststart
            if room >= 3:
                # It's a whitespace character at end-of-line, and we have room
                # for the three-character quoted encoding.
                q = quote(line[-1])
            elif room == 2:
                # There's room for the whitespace character and a soft break.
                q = line[-1] + soft_break
            else:
                # There's room only for a soft break.  The quoted whitespace
                # will be the only content on the subsequent line.
                q = soft_break + quote(line[-1])
            append(line[start:-1] + q)
        else:
            append(line[start:])

    # add back final newline if present
    if body[-1] in CRLF:
        append('')

    return eol.join(encoded_body)



# BAW: I'm not sure if the intent was for the signature of this function to be
# the same as base64MIME.decode() or not...
def decode(encoded, eol=NL):
    """Decode a quoted-printable string.

    Lines are separated with eol, which defaults to \\n.
    """
    if not encoded:
        return encoded
    # BAW: see comment in encode() above.  Again, we're building up the
    # decoded string with string concatenation, which could be done much more
    # efficiently.
    decoded = ''

    for line in encoded.splitlines():
        line = line.rstrip()
        if not line:
            decoded += eol
            continue

        i = 0
        n = len(line)
        while i < n:
            c = line[i]
            if c != '=':
                decoded += c
                i += 1
            # Otherwise, c == "=".  Are we at the end of the line?  If so, add
            # a soft line break.
            elif i+1 == n:
                i += 1
                continue
            # Decode if in form =AB
            elif i+2 < n and line[i+1] in hexdigits and line[i+2] in hexdigits:
                decoded += unquote(line[i:i+3])
                i += 3
            # Otherwise, not in form =AB, pass literally
            else:
                decoded += c
                i += 1

            if i == n:
                decoded += eol
    # Special case if original string did not end with eol
    if encoded[-1] not in '\r\n' and decoded.endswith(eol):
        decoded = decoded[:-len(eol)]
    return decoded


# For convenience and backwards compatibility w/ standard base64 module
body_decode = decode
decodestring = decode



def _unquote_match(match):
    """Turn a match in the form =AB to the ASCII character with value 0xab"""
    s = match.group(0)
    return unquote(s)


# Header decoding is done a bit differently
def header_decode(s):
    """Decode a string encoded with RFC 2045 MIME header `Q' encoding.

    This function does not parse a full MIME header value encoded with
    quoted-printable (like =?iso-8859-1?q?Hello_World?=) -- please use
    the high level email.header class for that functionality.
    """
    s = s.replace('_', ' ')
    return re.sub(r'=[a-fA-F0-9]{2}', _unquote_match, s, flags=re.ASCII)



# Important: don't add things to this module, as they will end up in the REPL's
# default globals.  Use _pyrepl.main instead.

# Avoid caching this file by linecache and incorrectly report tracebacks.
# See https://github.com/python/cpython/issues/129098.
__spec__ = __loader__ = None

if __name__ == "__main__":
    from .main import interactive_console as __pyrepl_interactive_console
    __pyrepl_interactive_console()



"""Event loop mixins."""

import threading
from . import events

_global_lock = threading.Lock()


class _LoopBoundMixin:
    _loop = None

    def _get_loop(self):
        loop = events._get_running_loop()

        if self._loop is None:
            with _global_lock:
                if self._loop is None:
                    self._loop = loop
        if loop is not self._loop:
            raise RuntimeError(f'{self!r} is bound to a different event loop')
        return loop



from __future__ import annotations

from importlib.metadata import version

__all__ = [
    "inject_into_urllib3",
    "extract_from_urllib3",
]

import typing

orig_HTTPSConnection: typing.Any = None


def inject_into_urllib3() -> None:
    # First check if h2 version is valid
    h2_version = version("h2")
    if not h2_version.startswith("4."):
        raise ImportError(
            "urllib3 v2 supports h2 version 4.x.x, currently "
            f"the 'h2' module is compiled with {h2_version!r}. "
            "See: https://github.com/urllib3/urllib3/issues/3290"
        )

    # Import here to avoid circular dependencies.
    from .. import connection as urllib3_connection
    from .. import util as urllib3_util
    from ..connectionpool import HTTPSConnectionPool
    from ..util import ssl_ as urllib3_util_ssl
    from .connection import HTTP2Connection

    global orig_HTTPSConnection
    orig_HTTPSConnection = urllib3_connection.HTTPSConnection

    HTTPSConnectionPool.ConnectionCls = HTTP2Connection
    urllib3_connection.HTTPSConnection = HTTP2Connection  # type: ignore[misc]

    # TODO: Offer 'http/1.1' as well, but for testing purposes this is handy.
    urllib3_util.ALPN_PROTOCOLS = ["h2"]
    urllib3_util_ssl.ALPN_PROTOCOLS = ["h2"]


def extract_from_urllib3() -> None:
    from .. import connection as urllib3_connection
    from .. import util as urllib3_util
    from ..connectionpool import HTTPSConnectionPool
    from ..util import ssl_ as urllib3_util_ssl

    HTTPSConnectionPool.ConnectionCls = orig_HTTPSConnection
    urllib3_connection.HTTPSConnection = orig_HTTPSConnection  # type: ignore[misc]

    urllib3_util.ALPN_PROTOCOLS = ["http/1.1"]
    urllib3_util_ssl.ALPN_PROTOCOLS = ["http/1.1"]



__all__ = 'iscoroutinefunction', 'iscoroutine'

import collections.abc
import inspect
import os
import sys
import types


def _is_debug_mode():
    # See: https://docs.python.org/3/library/asyncio-dev.html#asyncio-debug-mode.
    return sys.flags.dev_mode or (not sys.flags.ignore_environment and
                                  bool(os.environ.get('PYTHONASYNCIODEBUG')))


# A marker for iscoroutinefunction.
_is_coroutine = object()


def iscoroutinefunction(func):
    """Return True if func is a decorated coroutine function."""
    return (inspect.iscoroutinefunction(func) or
            getattr(func, '_is_coroutine', None) is _is_coroutine)


# Prioritize native coroutine check to speed-up
# asyncio.iscoroutine.
_COROUTINE_TYPES = (types.CoroutineType, collections.abc.Coroutine)
_iscoroutine_typecache = set()


def iscoroutine(obj):
    """Return True if obj is a coroutine object."""
    if type(obj) in _iscoroutine_typecache:
        return True

    if isinstance(obj, _COROUTINE_TYPES):
        # Just in case we don't want to cache more than 100
        # positive types.  That shouldn't ever happen, unless
        # someone stressing the system on purpose.
        if len(_iscoroutine_typecache) < 100:
            _iscoroutine_typecache.add(type(obj))
        return True
    else:
        return False


def _format_coroutine(coro):
    assert iscoroutine(coro)

    def get_name(coro):
        # Coroutines compiled with Cython sometimes don't have
        # proper __qualname__ or __name__.  While that is a bug
        # in Cython, asyncio shouldn't crash with an AttributeError
        # in its __repr__ functions.
        if hasattr(coro, '__qualname__') and coro.__qualname__:
            coro_name = coro.__qualname__
        elif hasattr(coro, '__name__') and coro.__name__:
            coro_name = coro.__name__
        else:
            # Stop masking Cython bugs, expose them in a friendly way.
            coro_name = f'<{type(coro).__name__} without __name__>'
        return f'{coro_name}()'

    def is_running(coro):
        try:
            return coro.cr_running
        except AttributeError:
            try:
                return coro.gi_running
            except AttributeError:
                return False

    coro_code = None
    if hasattr(coro, 'cr_code') and coro.cr_code:
        coro_code = coro.cr_code
    elif hasattr(coro, 'gi_code') and coro.gi_code:
        coro_code = coro.gi_code

    coro_name = get_name(coro)

    if not coro_code:
        # Built-in types might not have __qualname__ or __name__.
        if is_running(coro):
            return f'{coro_name} running'
        else:
            return coro_name

    coro_frame = None
    if hasattr(coro, 'gi_frame') and coro.gi_frame:
        coro_frame = coro.gi_frame
    elif hasattr(coro, 'cr_frame') and coro.cr_frame:
        coro_frame = coro.cr_frame

    # If Cython's coroutine has a fake code object without proper
    # co_filename -- expose that.
    filename = coro_code.co_filename or '<empty co_filename>'

    lineno = 0

    if coro_frame is not None:
        lineno = coro_frame.f_lineno
        coro_repr = f'{coro_name} running at {filename}:{lineno}'

    else:
        lineno = coro_code.co_firstlineno
        coro_repr = f'{coro_name} done, defined at {filename}:{lineno}'

    return coro_repr



from __future__ import annotations

import dataclasses
import logging
import re
from collections.abc import Mapping, Sequence
from dataclasses import dataclass
from datetime import datetime
from typing import (
    TYPE_CHECKING,
    Any,
    Callable,
    Protocol,
    TypeVar,
    cast,
)
from urllib.parse import urlparse

from .markers import Environment, Marker, default_environment
from .specifiers import SpecifierSet
from .tags import create_compatible_tags_selector, sys_tags
from .utils import (
    NormalizedName,
    is_normalized_name,
    parse_sdist_filename,
    parse_wheel_filename,
)
from .version import Version

if TYPE_CHECKING:  # pragma: no cover
    from collections.abc import Collection, Iterator
    from pathlib import Path

    from typing_extensions import Self

    from .tags import Tag

_logger = logging.getLogger(__name__)

__all__ = [
    "Package",
    "PackageArchive",
    "PackageDirectory",
    "PackageSdist",
    "PackageVcs",
    "PackageWheel",
    "Pylock",
    "PylockUnsupportedVersionError",
    "PylockValidationError",
    "is_valid_pylock_path",
]


def __dir__() -> list[str]:
    return __all__


_T = TypeVar("_T")
_T2 = TypeVar("_T2")


class _FromMappingProtocol(Protocol):  # pragma: no cover
    @classmethod
    def _from_dict(cls, d: Mapping[str, Any]) -> Self: ...


_FromMappingProtocolT = TypeVar("_FromMappingProtocolT", bound=_FromMappingProtocol)


_PYLOCK_FILE_NAME_RE = re.compile(r"^pylock\.([^.]+)\.toml$")


def is_valid_pylock_path(path: Path) -> bool:
    """Check if the given path is a valid pylock file path."""
    return path.name == "pylock.toml" or bool(_PYLOCK_FILE_NAME_RE.match(path.name))


def _toml_key(key: str) -> str:
    return key.replace("_", "-")


def _toml_value(key: str, value: Any) -> Any:  # noqa: ANN401
    if isinstance(value, (Version, Marker, SpecifierSet)):
        return str(value)
    if isinstance(value, Sequence) and key == "environments":
        return [str(v) for v in value]
    return value


def _toml_dict_factory(data: list[tuple[str, Any]]) -> dict[str, Any]:
    return {
        _toml_key(key): _toml_value(key, value)
        for key, value in data
        if value is not None
    }


def _get(d: Mapping[str, Any], expected_type: type[_T], key: str) -> _T | None:
    """Get a value from the dictionary and verify it's the expected type."""
    if (value := d.get(key)) is None:
        return None
    if not isinstance(value, expected_type):
        raise PylockValidationError(
            f"Unexpected type {type(value).__name__} "
            f"(expected {expected_type.__name__})",
            context=key,
        )
    return value


def _get_required(d: Mapping[str, Any], expected_type: type[_T], key: str) -> _T:
    """Get a required value from the dictionary and verify it's the expected type."""
    if (value := _get(d, expected_type, key)) is None:
        raise _PylockRequiredKeyError(key)
    return value


def _get_sequence(
    d: Mapping[str, Any], expected_item_type: type[_T], key: str
) -> Sequence[_T] | None:
    """Get a list value from the dictionary and verify it's the expected items type."""
    if (value := _get(d, Sequence, key)) is None:  # type: ignore[type-abstract]
        return None
    if isinstance(value, (str, bytes)):
        # special case: str and bytes are Sequences, but we want to reject it
        raise PylockValidationError(
            f"Unexpected type {type(value).__name__} (expected Sequence)",
            context=key,
        )
    for i, item in enumerate(value):
        if not isinstance(item, expected_item_type):
            raise PylockValidationError(
                f"Unexpected type {type(item).__name__} "
                f"(expected {expected_item_type.__name__})",
                context=f"{key}[{i}]",
            )
    return value


def _get_as(
    d: Mapping[str, Any],
    expected_type: type[_T],
    target_type: Callable[[_T], _T2],
    key: str,
) -> _T2 | None:
    """Get a value from the dictionary, verify it's the expected type,
    and convert to the target type.

    This assumes the target_type constructor accepts the value.
    """
    if (value := _get(d, expected_type, key)) is None:
        return None
    try:
        return target_type(value)
    except Exception as e:
        raise PylockValidationError(e, context=key) from e


def _get_required_as(
    d: Mapping[str, Any],
    expected_type: type[_T],
    target_type: Callable[[_T], _T2],
    key: str,
) -> _T2:
    """Get a required value from the dict, verify it's the expected type,
    and convert to the target type."""
    if (value := _get_as(d, expected_type, target_type, key)) is None:
        raise _PylockRequiredKeyError(key)
    return value


def _get_sequence_as(
    d: Mapping[str, Any],
    expected_item_type: type[_T],
    target_item_type: Callable[[_T], _T2],
    key: str,
) -> list[_T2] | None:
    """Get list value from dictionary and verify expected items type."""
    if (value := _get_sequence(d, expected_item_type, key)) is None:
        return None
    result = []
    try:
        for item in value:
            typed_item = target_item_type(item)
            result.append(typed_item)
    except Exception as e:
        raise PylockValidationError(e, context=f"{key}[{len(result)}]") from e
    return result


def _get_object(
    d: Mapping[str, Any], target_type: type[_FromMappingProtocolT], key: str
) -> _FromMappingProtocolT | None:
    """Get a dictionary value from the dictionary and convert it to a dataclass."""
    if (value := _get(d, Mapping, key)) is None:  # type: ignore[type-abstract]
        return None
    try:
        return target_type._from_dict(value)
    except Exception as e:
        raise PylockValidationError(e, context=key) from e


def _get_sequence_of_objects(
    d: Mapping[str, Any], target_item_type: type[_FromMappingProtocolT], key: str
) -> list[_FromMappingProtocolT] | None:
    """Get a list value from the dictionary and convert its items to a dataclass."""
    if (value := _get_sequence(d, Mapping, key)) is None:  # type: ignore[type-abstract]
        return None
    result: list[_FromMappingProtocolT] = []
    try:
        for item in value:
            typed_item = target_item_type._from_dict(item)
            result.append(typed_item)
    except Exception as e:
        raise PylockValidationError(e, context=f"{key}[{len(result)}]") from e
    return result


def _get_required_sequence_of_objects(
    d: Mapping[str, Any], target_item_type: type[_FromMappingProtocolT], key: str
) -> Sequence[_FromMappingProtocolT]:
    """Get a required list value from the dictionary and convert its items to a
    dataclass."""
    if (result := _get_sequence_of_objects(d, target_item_type, key)) is None:
        raise _PylockRequiredKeyError(key)
    return result


def _validate_normalized_name(name: str) -> NormalizedName:
    """Validate that a string is a NormalizedName."""
    if not is_normalized_name(name):
        raise PylockValidationError(f"Name {name!r} is not normalized")
    return NormalizedName(name)


def _validate_path_url(path: str | None, url: str | None) -> None:
    if not path and not url:
        raise PylockValidationError("path or url must be provided")


def _path_name(path: str | None) -> str | None:
    if not path:
        return None
    # If the path is relative it MAY use POSIX-style path separators explicitly
    # for portability
    if "/" in path:
        return path.rsplit("/", 1)[-1]
    elif "\\" in path:
        return path.rsplit("\\", 1)[-1]
    else:
        return path


def _url_name(url: str | None) -> str | None:
    if not url:
        return None
    url_path = urlparse(url).path
    return url_path.rsplit("/", 1)[-1]


def _validate_hashes(hashes: Mapping[str, Any]) -> Mapping[str, Any]:
    if not hashes:
        raise PylockValidationError("At least one hash must be provided")
    if not all(isinstance(hash_val, str) for hash_val in hashes.values()):
        raise PylockValidationError("Hash values must be strings")
    return hashes


class PylockValidationError(Exception):
    """Raised when when input data is not spec-compliant."""

    context: str | None = None
    message: str

    def __init__(
        self,
        cause: str | Exception,
        *,
        context: str | None = None,
    ) -> None:
        if isinstance(cause, PylockValidationError):
            if cause.context:
                self.context = (
                    f"{context}.{cause.context}" if context else cause.context
                )
            else:
                self.context = context
            self.message = cause.message
        else:
            self.context = context
            self.message = str(cause)

    def __str__(self) -> str:
        if self.context:
            return f"{self.message} in {self.context!r}"
        return self.message


class _PylockRequiredKeyError(PylockValidationError):
    def __init__(self, key: str) -> None:
        super().__init__("Missing required value", context=key)


class PylockUnsupportedVersionError(PylockValidationError):
    """Raised when encountering an unsupported `lock_version`."""


class PylockSelectError(Exception):
    """Base exception for errors raised by :meth:`Pylock.select`."""


@dataclass(frozen=True, init=False)
class PackageVcs:
    type: str
    url: str | None = None
    path: str | None = None
    requested_revision: str | None = None
    commit_id: str  # type: ignore[misc]
    subdirectory: str | None = None

    def __init__(
        self,
        *,
        type: str,
        url: str | None = None,
        path: str | None = None,
        requested_revision: str | None = None,
        commit_id: str,
        subdirectory: str | None = None,
    ) -> None:
        # In Python 3.10+ make dataclass kw_only=True and remove __init__
        object.__setattr__(self, "type", type)
        object.__setattr__(self, "url", url)
        object.__setattr__(self, "path", path)
        object.__setattr__(self, "requested_revision", requested_revision)
        object.__setattr__(self, "commit_id", commit_id)
        object.__setattr__(self, "subdirectory", subdirectory)

    @classmethod
    def _from_dict(cls, d: Mapping[str, Any]) -> Self:
        package_vcs = cls(
            type=_get_required(d, str, "type"),
            url=_get(d, str, "url"),
            path=_get(d, str, "path"),
            requested_revision=_get(d, str, "requested-revision"),
            commit_id=_get_required(d, str, "commit-id"),
            subdirectory=_get(d, str, "subdirectory"),
        )
        _validate_path_url(package_vcs.path, package_vcs.url)
        return package_vcs


@dataclass(frozen=True, init=False)
class PackageDirectory:
    path: str
    editable: bool | None = None
    subdirectory: str | None = None

    def __init__(
        self,
        *,
        path: str,
        editable: bool | None = None,
        subdirectory: str | None = None,
    ) -> None:
        # In Python 3.10+ make dataclass kw_only=True and remove __init__
        object.__setattr__(self, "path", path)
        object.__setattr__(self, "editable", editable)
        object.__setattr__(self, "subdirectory", subdirectory)

    @classmethod
    def _from_dict(cls, d: Mapping[str, Any]) -> Self:
        return cls(
            path=_get_required(d, str, "path"),
            editable=_get(d, bool, "editable"),
            subdirectory=_get(d, str, "subdirectory"),
        )


@dataclass(frozen=True, init=False)
class PackageArchive:
    url: str | None = None
    path: str | None = None
    size: int | None = None
    upload_time: datetime | None = None
    hashes: Mapping[str, str]  # type: ignore[misc]
    subdirectory: str | None = None

    def __init__(
        self,
        *,
        url: str | None = None,
        path: str | None = None,
        size: int | None = None,
        upload_time: datetime | None = None,
        hashes: Mapping[str, str],
        subdirectory: str | None = None,
    ) -> None:
        # In Python 3.10+ make dataclass kw_only=True and remove __init__
        object.__setattr__(self, "url", url)
        object.__setattr__(self, "path", path)
        object.__setattr__(self, "size", size)
        object.__setattr__(self, "upload_time", upload_time)
        object.__setattr__(self, "hashes", hashes)
        object.__setattr__(self, "subdirectory", subdirectory)

    @classmethod
    def _from_dict(cls, d: Mapping[str, Any]) -> Self:
        package_archive = cls(
            url=_get(d, str, "url"),
            path=_get(d, str, "path"),
            size=_get(d, int, "size"),
            upload_time=_get(d, datetime, "upload-time"),
            hashes=_get_required_as(d, Mapping, _validate_hashes, "hashes"),  # type: ignore[type-abstract]
            subdirectory=_get(d, str, "subdirectory"),
        )
        _validate_path_url(package_archive.path, package_archive.url)
        return package_archive


@dataclass(frozen=True, init=False)
class PackageSdist:
    name: str | None = None
    upload_time: datetime | None = None
    url: str | None = None
    path: str | None = None
    size: int | None = None
    hashes: Mapping[str, str]  # type: ignore[misc]

    def __init__(
        self,
        *,
        name: str | None = None,
        upload_time: datetime | None = None,
        url: str | None = None,
        path: str | None = None,
        size: int | None = None,
        hashes: Mapping[str, str],
    ) -> None:
        # In Python 3.10+ make dataclass kw_only=True and remove __init__
        object.__setattr__(self, "name", name)
        object.__setattr__(self, "upload_time", upload_time)
        object.__setattr__(self, "url", url)
        object.__setattr__(self, "path", path)
        object.__setattr__(self, "size", size)
        object.__setattr__(self, "hashes", hashes)

    @classmethod
    def _from_dict(cls, d: Mapping[str, Any]) -> Self:
        package_sdist = cls(
            name=_get(d, str, "name"),
            upload_time=_get(d, datetime, "upload-time"),
            url=_get(d, str, "url"),
            path=_get(d, str, "path"),
            size=_get(d, int, "size"),
            hashes=_get_required_as(d, Mapping, _validate_hashes, "hashes"),  # type: ignore[type-abstract]
        )
        _validate_path_url(package_sdist.path, package_sdist.url)
        return package_sdist

    @property
    def filename(self) -> str:
        """Get the filename of the sdist."""
        filename = self.name or _path_name(self.path) or _url_name(self.url)
        if not filename:
            raise PylockValidationError("Cannot determine sdist filename")
        return filename


@dataclass(frozen=True, init=False)
class PackageWheel:
    name: str | None = None
    upload_time: datetime | None = None
    url: str | None = None
    path: str | None = None
    size: int | None = None
    hashes: Mapping[str, str]  # type: ignore[misc]

    def __init__(
        self,
        *,
        name: str | None = None,
        upload_time: datetime | None = None,
        url: str | None = None,
        path: str | None = None,
        size: int | None = None,
        hashes: Mapping[str, str],
    ) -> None:
        # In Python 3.10+ make dataclass kw_only=True and remove __init__
        object.__setattr__(self, "name", name)
        object.__setattr__(self, "upload_time", upload_time)
        object.__setattr__(self, "url", url)
        object.__setattr__(self, "path", path)
        object.__setattr__(self, "size", size)
        object.__setattr__(self, "hashes", hashes)

    @classmethod
    def _from_dict(cls, d: Mapping[str, Any]) -> Self:
        package_wheel = cls(
            name=_get(d, str, "name"),
            upload_time=_get(d, datetime, "upload-time"),
            url=_get(d, str, "url"),
            path=_get(d, str, "path"),
            size=_get(d, int, "size"),
            hashes=_get_required_as(d, Mapping, _validate_hashes, "hashes"),  # type: ignore[type-abstract]
        )
        _validate_path_url(package_wheel.path, package_wheel.url)
        return package_wheel

    @property
    def filename(self) -> str:
        """Get the filename of the wheel."""
        filename = self.name or _path_name(self.path) or _url_name(self.url)
        if not filename:
            raise PylockValidationError("Cannot determine wheel filename")
        return filename


@dataclass(frozen=True, init=False)
class Package:
    name: NormalizedName
    version: Version | None = None
    marker: Marker | None = None
    requires_python: SpecifierSet | None = None
    dependencies: Sequence[Mapping[str, Any]] | None = None
    vcs: PackageVcs | None = None
    directory: PackageDirectory | None = None
    archive: PackageArchive | None = None
    index: str | None = None
    sdist: PackageSdist | None = None
    wheels: Sequence[PackageWheel] | None = None
    attestation_identities: Sequence[Mapping[str, Any]] | None = None
    tool: Mapping[str, Any] | None = None

    def __init__(
        self,
        *,
        name: NormalizedName,
        version: Version | None = None,
        marker: Marker | None = None,
        requires_python: SpecifierSet | None = None,
        dependencies: Sequence[Mapping[str, Any]] | None = None,
        vcs: PackageVcs | None = None,
        directory: PackageDirectory | None = None,
        archive: PackageArchive | None = None,
        index: str | None = None,
        sdist: PackageSdist | None = None,
        wheels: Sequence[PackageWheel] | None = None,
        attestation_identities: Sequence[Mapping[str, Any]] | None = None,
        tool: Mapping[str, Any] | None = None,
    ) -> None:
        # In Python 3.10+ make dataclass kw_only=True and remove __init__
        object.__setattr__(self, "name", name)
        object.__setattr__(self, "version", version)
        object.__setattr__(self, "marker", marker)
        object.__setattr__(self, "requires_python", requires_python)
        object.__setattr__(self, "dependencies", dependencies)
        object.__setattr__(self, "vcs", vcs)
        object.__setattr__(self, "directory", directory)
        object.__setattr__(self, "archive", archive)
        object.__setattr__(self, "index", index)
        object.__setattr__(self, "sdist", sdist)
        object.__setattr__(self, "wheels", wheels)
        object.__setattr__(self, "attestation_identities", attestation_identities)
        object.__setattr__(self, "tool", tool)

    @classmethod
    def _from_dict(cls, d: Mapping[str, Any]) -> Self:
        package = cls(
            name=_get_required_as(d, str, _validate_normalized_name, "name"),
            version=_get_as(d, str, Version, "version"),
            requires_python=_get_as(d, str, SpecifierSet, "requires-python"),
            dependencies=_get_sequence(d, Mapping, "dependencies"),  # type: ignore[type-abstract]
            marker=_get_as(d, str, Marker, "marker"),
            vcs=_get_object(d, PackageVcs, "vcs"),
            directory=_get_object(d, PackageDirectory, "directory"),
            archive=_get_object(d, PackageArchive, "archive"),
            index=_get(d, str, "index"),
            sdist=_get_object(d, PackageSdist, "sdist"),
            wheels=_get_sequence_of_objects(d, PackageWheel, "wheels"),
            attestation_identities=_get_sequence(d, Mapping, "attestation-identities"),  # type: ignore[type-abstract]
            tool=_get(d, Mapping, "tool"),  # type: ignore[type-abstract]
        )
        distributions = bool(package.sdist) + len(package.wheels or [])
        direct_urls = (
            bool(package.vcs) + bool(package.directory) + bool(package.archive)
        )
        if distributions > 0 and direct_urls > 0:
            raise PylockValidationError(
                "None of vcs, directory, archive must be set if sdist or wheels are set"
            )
        if distributions == 0 and direct_urls != 1:
            raise PylockValidationError(
                "Exactly one of vcs, directory, archive must be set "
                "if sdist and wheels are not set"
            )
        for i, wheel in enumerate(package.wheels or []):
            try:
                (name, version, _, _) = parse_wheel_filename(wheel.filename)
            except Exception as e:
                raise PylockValidationError(
                    f"Invalid wheel filename {wheel.filename!r}",
                    context=f"wheels[{i}]",
                ) from e
            if name != package.name:
                raise PylockValidationError(
                    f"Name in {wheel.filename!r} is not consistent with "
                    f"package name {package.name!r}",
                    context=f"wheels[{i}]",
                )
            if package.version and version != package.version:
                raise PylockValidationError(
                    f"Version in {wheel.filename!r} is not consistent with "
                    f"package version {str(package.version)!r}",
                    context=f"wheels[{i}]",
                )
        if package.sdist:
            try:
                name, version = parse_sdist_filename(package.sdist.filename)
            except Exception as e:
                raise PylockValidationError(
                    f"Invalid sdist filename {package.sdist.filename!r}",
                    context="sdist",
                ) from e
            if name != package.name:
                raise PylockValidationError(
                    f"Name in {package.sdist.filename!r} is not consistent with "
                    f"package name {package.name!r}",
                    context="sdist",
                )
            if package.version and version != package.version:
                raise PylockValidationError(
                    f"Version in {package.sdist.filename!r} is not consistent with "
                    f"package version {str(package.version)!r}",
                    context="sdist",
                )
        try:
            for i, attestation_identity in enumerate(  # noqa: B007
                package.attestation_identities or []
            ):
                _get_required(attestation_identity, str, "kind")
        except Exception as e:
            raise PylockValidationError(
                e, context=f"attestation-identities[{i}]"
            ) from e
        return package

    @property
    def is_direct(self) -> bool:
        return not (self.sdist or self.wheels)


@dataclass(frozen=True, init=False)
class Pylock:
    """A class representing a pylock file."""

    lock_version: Version
    environments: Sequence[Marker] | None = None
    requires_python: SpecifierSet | None = None
    extras: Sequence[NormalizedName] | None = None
    dependency_groups: Sequence[str] | None = None
    default_groups: Sequence[str] | None = None
    created_by: str  # type: ignore[misc]
    packages: Sequence[Package]  # type: ignore[misc]
    tool: Mapping[str, Any] | None = None

    def __init__(
        self,
        *,
        lock_version: Version,
        environments: Sequence[Marker] | None = None,
        requires_python: SpecifierSet | None = None,
        extras: Sequence[NormalizedName] | None = None,
        dependency_groups: Sequence[str] | None = None,
        default_groups: Sequence[str] | None = None,
        created_by: str,
        packages: Sequence[Package],
        tool: Mapping[str, Any] | None = None,
    ) -> None:
        # In Python 3.10+ make dataclass kw_only=True and remove __init__
        object.__setattr__(self, "lock_version", lock_version)
        object.__setattr__(self, "environments", environments)
        object.__setattr__(self, "requires_python", requires_python)
        object.__setattr__(self, "extras", extras)
        object.__setattr__(self, "dependency_groups", dependency_groups)
        object.__setattr__(self, "default_groups", default_groups)
        object.__setattr__(self, "created_by", created_by)
        object.__setattr__(self, "packages", packages)
        object.__setattr__(self, "tool", tool)

    @classmethod
    def _from_dict(cls, d: Mapping[str, Any]) -> Self:
        pylock = cls(
            lock_version=_get_required_as(d, str, Version, "lock-version"),
            environments=_get_sequence_as(d, str, Marker, "environments"),
            extras=_get_sequence_as(d, str, _validate_normalized_name, "extras"),
            dependency_groups=_get_sequence(d, str, "dependency-groups"),
            default_groups=_get_sequence(d, str, "default-groups"),
            created_by=_get_required(d, str, "created-by"),
            requires_python=_get_as(d, str, SpecifierSet, "requires-python"),
            packages=_get_required_sequence_of_objects(d, Package, "packages"),
            tool=_get(d, Mapping, "tool"),  # type: ignore[type-abstract]
        )
        if not Version("1") <= pylock.lock_version < Version("2"):
            raise PylockUnsupportedVersionError(
                f"pylock version {pylock.lock_version} is not supported"
            )
        if pylock.lock_version > Version("1.0"):
            _logger.warning(
                "pylock minor version %s is not supported", pylock.lock_version
            )
        return pylock

    @classmethod
    def from_dict(cls, d: Mapping[str, Any], /) -> Self:
        """Create and validate a Pylock instance from a TOML dictionary.

        Raises :class:`PylockValidationError` if the input data is not
        spec-compliant.
        """
        return cls._from_dict(d)

    def to_dict(self) -> Mapping[str, Any]:
        """Convert the Pylock instance to a TOML dictionary."""
        return dataclasses.asdict(self, dict_factory=_toml_dict_factory)

    def validate(self) -> None:
        """Validate the Pylock instance against the specification.

        Raises :class:`PylockValidationError` otherwise."""
        self.from_dict(self.to_dict())

    def select(
        self,
        *,
        environment: Environment | None = None,
        tags: Sequence[Tag] | None = None,
        extras: Collection[str] | None = None,
        dependency_groups: Collection[str] | None = None,
    ) -> Iterator[
        tuple[
            Package,
            PackageVcs
            | PackageDirectory
            | PackageArchive
            | PackageWheel
            | PackageSdist,
        ]
    ]:
        """Select what to install from the lock file.

        The *environment* and *tags* parameters represent the environment being
        selected for. If unspecified, ``packaging.markers.default_environment()`` and
        ``packaging.tags.sys_tags()`` are used.

        The *extras* parameter represents the extras to install.

        The *dependency_groups* parameter represents the groups to install. If
        unspecified, the default groups are used.

        This method must be used on valid Pylock instances (i.e. one obtained
        from :meth:`Pylock.from_dict` or if constructed manually, after calling
        :meth:`Pylock.validate`).
        """
        compatible_tags_selector = create_compatible_tags_selector(tags or sys_tags())

        # #. Gather the extras and dependency groups to install and set ``extras`` and
        #    ``dependency_groups`` for marker evaluation, respectively.
        #
        #    #. ``extras`` SHOULD be set to the empty set by default.
        #    #. ``dependency_groups`` SHOULD be the set created from
        #       :ref:`pylock-default-groups` by default.
        env = cast(
            "dict[str, str | frozenset[str]]",
            dict(
                environment or {},  # Marker.evaluate will fill-up
                extras=frozenset(extras or []),
                dependency_groups=frozenset(
                    (self.default_groups or [])
                    if dependency_groups is None  # to allow selecting no group
                    else dependency_groups
                ),
            ),
        )
        env_python_full_version = (
            environment["python_full_version"]
            if environment
            else default_environment()["python_full_version"]
        )

        # #. Check if the metadata version specified by :ref:`pylock-lock-version` is
        #    supported; an error or warning MUST be raised as appropriate.
        # Covered by lock.validate() which is a precondition for this method.

        # #. If :ref:`pylock-requires-python` is specified, check that the environment
        #    being installed for meets the requirement; an error MUST be raised if it is
        #    not met.
        if self.requires_python and not self.requires_python.contains(
            env_python_full_version,
        ):
            raise PylockSelectError(
                f"python_full_version {env_python_full_version!r} "
                f"in provided environment does not satisfy the Python version "
                f"requirement {str(self.requires_python)!r}"
            )

        # #. If :ref:`pylock-environments` is specified, check that at least one of the
        #    environment marker expressions is satisfied; an error MUST be raised if no
        #    expression is satisfied.
        if self.environments:
            for env_marker in self.environments:
                if env_marker.evaluate(
                    cast("dict[str, str]", environment or {}), context="requirement"
                ):
                    break
            else:
                raise PylockSelectError(
                    "Provided environment does not satisfy any of the "
                    "environments specified in the lock file"
                )

        # #. For each package listed in :ref:`pylock-packages`:
        selected_packages_by_name: dict[str, tuple[int, Package]] = {}
        for package_index, package in enumerate(self.packages):
            # #. If :ref:`pylock-packages-marker` is specified, check if it is
            #    satisfied;if it isn't, skip to the next package.
            if package.marker and not package.marker.evaluate(env, context="lock_file"):
                continue

            # #. If :ref:`pylock-packages-requires-python` is specified, check if it is
            #    satisfied; an error MUST be raised if it isn't.
            if package.requires_python and not package.requires_python.contains(
                env_python_full_version,
            ):
                raise PylockSelectError(
                    f"python_full_version {env_python_full_version!r} "
                    f"in provided environment does not satisfy the Python version "
                    f"requirement {str(package.requires_python)!r} for package "
                    f"{package.name!r} at packages[{package_index}]"
                )

            # #. Check that no other conflicting instance of the package has been slated
            #    to be installed; an error about the ambiguity MUST be raised otherwise.
            if package.name in selected_packages_by_name:
                raise PylockSelectError(
                    f"Multiple packages with the name {package.name!r} are "
                    f"selected at packages[{package_index}] and "
                    f"packages[{selected_packages_by_name[package.name][0]}]"
                )

            # #. Check that the source of the package is specified appropriately (i.e.
            #    there are no conflicting sources in the package entry);
            #    an error MUST be raised if any issues are found.
            # Covered by lock.validate() which is a precondition for this method.

            # #. Add the package to the set of packages to install.
            selected_packages_by_name[package.name] = (package_index, package)

        # #. For each package to be installed:
        for package_index, package in selected_packages_by_name.values():
            # - If :ref:`pylock-packages-vcs` is set:
            if package.vcs is not None:
                yield package, package.vcs

            # - Else if :ref:`pylock-packages-directory` is set:
            elif package.directory is not None:
                yield package, package.directory

            # - Else if :ref:`pylock-packages-archive` is set:
            elif package.archive is not None:
                yield package, package.archive

            # - Else if there are entries for :ref:`pylock-packages-wheels`:
            elif package.wheels:
                # #. Look for the appropriate wheel file based on
                #    :ref:`pylock-packages-wheels-name`; if one is not found then move
                #    on to :ref:`pylock-packages-sdist` or an error MUST be raised about
                #    a lack of source for the project.
                best_wheel = next(
                    compatible_tags_selector(
                        (wheel, parse_wheel_filename(wheel.filename)[-1])
                        for wheel in package.wheels
                    ),
                    None,
                )
                if best_wheel:
                    yield package, best_wheel
                elif package.sdist is not None:
                    yield package, package.sdist
                else:
                    raise PylockSelectError(
                        f"No wheel found matching the provided tags "
                        f"for package {package.name!r} "
                        f"at packages[{package_index}], "
                        f"and no sdist available as a fallback"
                    )

            # - Else if no :ref:`pylock-packages-wheels` file is found or
            #   :ref:`pylock-packages-sdist` is solely set:
            elif package.sdist is not None:
                yield package, package.sdist

            else:
                # Covered by lock.validate() which is a precondition for this method.
                raise NotImplementedError  # pragma: no cover



import struct


def load_tzdata(key):
    from importlib import resources

    components = key.split("/")
    package_name = ".".join(["tzdata.zoneinfo"] + components[:-1])
    resource_name = components[-1]

    try:
        path = resources.files(package_name).joinpath(resource_name)
        # gh-85702: Prevent PermissionError on Windows
        if path.is_dir():
            raise IsADirectoryError
        return path.open("rb")
    except (ImportError, FileNotFoundError, UnicodeEncodeError, IsADirectoryError):
        # There are four types of exception that can be raised that all amount
        # to "we cannot find this key":
        #
        # ImportError: If package_name doesn't exist (e.g. if tzdata is not
        #   installed, or if there's an error in the folder name like
        #   Amrica/New_York)
        # FileNotFoundError: If resource_name doesn't exist in the package
        #   (e.g. Europe/Krasnoy)
        # UnicodeEncodeError: If package_name or resource_name are not UTF-8,
        #   such as keys containing a surrogate character.
        # IsADirectoryError: If package_name without a resource_name specified.
        raise ZoneInfoNotFoundError(f"No time zone found with key {key}")


def load_data(fobj):
    header = _TZifHeader.from_file(fobj)

    if header.version == 1:
        time_size = 4
        time_type = "l"
    else:
        # Version 2+ has 64-bit integer transition times
        time_size = 8
        time_type = "q"

        # Version 2+ also starts with a Version 1 header and data, which
        # we need to skip now
        skip_bytes = (
            header.timecnt * 5  # Transition times and types
            + header.typecnt * 6  # Local time type records
            + header.charcnt  # Time zone designations
            + header.leapcnt * 8  # Leap second records
            + header.isstdcnt  # Standard/wall indicators
            + header.isutcnt  # UT/local indicators
        )

        fobj.seek(skip_bytes, 1)

        # Now we need to read the second header, which is not the same
        # as the first
        header = _TZifHeader.from_file(fobj)

    typecnt = header.typecnt
    timecnt = header.timecnt
    charcnt = header.charcnt

    # The data portion starts with timecnt transitions and indices
    if timecnt:
        trans_list_utc = struct.unpack(
            f">{timecnt}{time_type}", fobj.read(timecnt * time_size)
        )
        trans_idx = struct.unpack(f">{timecnt}B", fobj.read(timecnt))

        if max(trans_idx) >= typecnt:
            raise ValueError("Invalid transition index found while reading TZif: "
                            f"{max(trans_idx)}")
    else:
        trans_list_utc = ()
        trans_idx = ()

    # Read the ttinfo struct, (utoff, isdst, abbrind)
    if typecnt:
        utcoff, isdst, abbrind = zip(
            *(struct.unpack(">lbb", fobj.read(6)) for i in range(typecnt))
        )
    else:
        utcoff = ()
        isdst = ()
        abbrind = ()

    # Now read the abbreviations. They are null-terminated strings, indexed
    # not by position in the array but by position in the unsplit
    # abbreviation string. I suppose this makes more sense in C, which uses
    # null to terminate the strings, but it's inconvenient here...
    abbr_vals = {}
    abbr_chars = fobj.read(charcnt)

    def get_abbr(idx):
        # Gets a string starting at idx and running until the next \x00
        #
        # We cannot pre-populate abbr_vals by splitting on \x00 because there
        # are some zones that use subsets of longer abbreviations, like so:
        #
        #  LMT\x00AHST\x00HDT\x00
        #
        # Where the idx to abbr mapping should be:
        #
        # {0: "LMT", 4: "AHST", 5: "HST", 9: "HDT"}
        if idx not in abbr_vals:
            span_end = abbr_chars.find(b"\x00", idx)
            abbr_vals[idx] = abbr_chars[idx:span_end].decode()

        return abbr_vals[idx]

    abbr = tuple(get_abbr(idx) for idx in abbrind)

    # The remainder of the file consists of leap seconds (currently unused) and
    # the standard/wall and ut/local indicators, which are metadata we don't need.
    # In version 2 files, we need to skip the unnecessary data to get at the TZ string:
    if header.version >= 2:
        # Each leap second record has size (time_size + 4)
        skip_bytes = header.isutcnt + header.isstdcnt + header.leapcnt * 12
        fobj.seek(skip_bytes, 1)

        c = fobj.read(1)  # Should be \n
        assert c == b"\n", c

        line = fobj.readline()
        if not line.endswith(b"\n"):
            raise ValueError("Invalid TZif file: unexpected end of file")
        tz_str = line.rstrip(b"\n")
    else:
        tz_str = None

    return trans_idx, trans_list_utc, utcoff, isdst, abbr, tz_str


class _TZifHeader:
    __slots__ = [
        "version",
        "isutcnt",
        "isstdcnt",
        "leapcnt",
        "timecnt",
        "typecnt",
        "charcnt",
    ]

    def __init__(self, *args):
        for attr, val in zip(self.__slots__, args, strict=True):
            setattr(self, attr, val)

    @classmethod
    def from_file(cls, stream):
        # The header starts with a 4-byte "magic" value
        if stream.read(4) != b"TZif":
            raise ValueError("Invalid TZif file: magic not found")

        _version = stream.read(1)
        if _version == b"\x00":
            version = 1
        else:
            version = int(_version)
        stream.read(15)

        args = (version,)

        # Slots are defined in the order that the bytes are arranged
        args = args + struct.unpack(">6l", stream.read(24))

        return cls(*args)


class ZoneInfoNotFoundError(KeyError):
    """Exception raised when a ZoneInfo key is not found."""
